####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
# Name: addboard.pm 
# Usage: module called in the backend
# Steps: +addboard.pm
#          +addboard()
#            -setup() 
#            -write_page() 
#            -error()
#            -template() 
# 
# Update: 09-04-02 Jerome - added require for contenttype.inc
#         08-20-02 Nick and Jerome - Substituting out language-dependent
#           text strings with variables to allow multilingual support.
#	  07-21-00 kenny replaced hardcoded references with customize vars
#         02-02-00 Yelena adding code to make a mail.txt file in the board
#           directory if the board if for a TA to contain the TAs email.   
#         10-3-99 Gavin - worked on bug 140
#           $dirname is no longer passed, because the front end does not
#           pass this variable to it anymore, rather it uses the title and 
#           does some string manipulation on it.
#         07-29-99 Yelena - will now update the boardnames table instead of the 
#           customboard.htm table
#         07-26-99 Yelena - creates file called customboard.htm to allow admin
#           of extra boards 
#         07-02-99 Yelena - will now update the boards table when a new
#           board is created. Does not have capability to add ta boards.  
#         02-01-99 mike - fixing bug where admin script is getting hardcoded wwwboard.
#           commented out DBI mysql connect because repeated calls were breaking
#           rabow.pl script. I don't know what it would be used for normally.
#         11-12-98 starting 
# NEED
#   - allow for title
#   - use same template models (ie: history and classweb's?)
#   - allow for authentication
#   - link back to class site?  no easy way if we don't know class
# ASSUMES
#   - this routine is called from stub.pl 
#   - directory gets setup under ../ 
# 
# Status: Production
# Purpose: sets up a ClassWeb discussion board
#          similar to clonebbs.pl but with new auth/non-auth version
#          should be called from stub.pl

######################################################
# Name: addboard
# Usage: command line: ($srs, $term, $dirname, $title)
# Update: 07-02-99 Yelena - changing $payload to reflect new classweb
#   directory structure
# Status: Production
# Purpose: main routine to add an extra board for a class 

## $BASE is set in Apache Environment with SetEnv and PassEnv in httpd.conf
$BASE = "/home/classweb"; # main classweb programs directory

require "$BASE/customize.pl";  # important vars set here
require "$BASE/dbvars.pl";     # database vars set here
require("$BASE/dbopen.pl");
sub addboard
{
# oldusage: addboard.pl term srs dirname "Title of Board"
# term and srs will be added to the auth.inc file so 
# that authentication can be done.

# print STDERR "TESTING Made it into addboard.pl\n";
####################################################
# set up database variables and connection

# location of model admin stub program
$stub = "$BASE/stub.pl"; 
# location of classbase
$classbase = "$DOCUMENT_ROOT";          
# location of material to be copied
$payload = "$BASE/prof/payload/wwwboard";
# location of material to be copied
$ta_payload = "$BASE/ta/payload/wwwboard";
# location of homework dir
$homework_dir = "$BASE/prof/homework";
# location of ta homework dir
$ta_homework_dir = "$BASE/ta/homework";
####################################################
#  handle command line parameters
$srs = param('srs');
$term = param('term');
$title = param('title');
$authenticate = param('authenticate');
$homework = param('homework');
$creator = param('creator');
$ta_uid = param('ta_uid');
if($ta_uid){
  $payload = $ta_payload;
  $homework_dir = $ta_homework_dir;
}

###Error checking the title ###
if($title eq ""){
  &error("$_EMPTYTITLEENTERED");
}
$temp = $title;
$title =~ s/\'/\\'/gm;
$search = "select directory from boardnames where term = '$term' && srs = '$srs' && title = '$title'";
$sth = $dbh->prepare($search) || die ("Couldn't prepare $search");
$sth->execute || die ("Couldn't execute $search");
$numrows = $sth->rows;
if($numrows != 0){
  &error("$_TITLEENTEREDEXISTS\n");
}
$title = $temp;
$dirname = $title;

# remove the word "the, that, which, of, if, as, a, is, this, with"
$dirname =~ s/\s*the \s*//igm;
$dirname =~ s/\s*that \s*//igm;
$dirname =~ s/\s*which \s*//igm;
$dirname =~ s/\s*of \s*//igm;
$dirname =~ s/\s*if \s*//igm;
$dirname =~ s/\s*as \s*//igm;
$dirname =~ s/\s*this \s*//igm;
$dirname =~ s/\s*a \s*//igm;
$dirname =~ s/\s*is \s*//igm;
$dirname =~ s/\s*with \s*//igm;
$dirname =~ s/\s* the\s*//igm;
$dirname =~ s/\s* that\s*//igm;
$dirname =~ s/\s* whichs*//igm;
$dirname =~ s/\s* of\s*//igm;
$dirname =~ s/\s* if\s*//igm;
$dirname =~ s/\s* this\s*//igm;
$dirname =~ s/\s* as\s*//igm;
$dirname =~ s/\s* a\s*//igm;
$dirname =~ s/\s* is\s*//igm;
$dirname =~ s/\s* with\s*//igm;

if( $dirname eq "" ){
  $dirname = "$title";
}

# remove empty spaces
$dirname =~ s/\s*//gm;
$dirname =~ s/([a-zA-Z_0-9]+)/$1/;
$dirname = $1;


#grab only the first 12 characters in order to make directory small
$dirname = unpack("A12", $dirname);

require "update_boards.pm";      #updates boards table when new board created
require "update_boardnames.pm";  #updates boardnames table when new board created

# groups which desire custom layouts; see note1
%customgroup = ( $CUSTOM_GROUP);

# location of custom models; see note1
%custompayload = ($CUSTOM_PAYLOAD);

$|++;     # unbuffers output   - maybe better in calling routine

####################################################
#  set up database variables and connection

my $database = "$dbname";
# my $dbuser   = "$dbuser";
my $dbpass   = "$dbpswd"; 
&setup;     # need error check here


################
# Subroutines  #
################

####################################################
# Name: setup
# Usage: &setup() 
# Update:
# Status: Production
# Purpose: calls sub write-page to created directory, write pages, and set
#   up bulletin board

sub setup {

  %FORM = ();    # make sure nothing carries over
  &write_page;   # create directory, write pages and setup bulletin board
  return;
} #setup


##################################################################
# Name: write_page
# Usage:  &write_page()
# Update: 07-02-99 Yelena - replaced $payload to reflect new directory
#   structure.  Added ability to update boards table with new board
# Status: production
# Purpose: creates a directory for new board, updates the boards table

sub write_page {

  if (!$ta_uid) {
    # location of payload data
    $payload = "$BASE/prof/payload/wwwboard"; 
  } else {
    # location of ta payload data
    $payload = "$BASE/ta/payload/wwwboard";
  }

  $outdir = "$basedir/$dirname";
  if ($outdir=~/\s/) {&error("$_DIRECTORYNOSPACES");}

  $count = 2;

  while (-e "$outdir") {

    # if it is not the first time in the loop, need to chop two characters off 
    if($count != 2) {
      chop($outdir);
      chop($outdir);
      chop($dirname);
      chop($dirname);
    }
  
  $outdir = $outdir . "_" . $count;
  $dirname = $dirname . "_" . $count;
  $count++;
  }
  $index=$dirname; 

  # check for overwrite of existing class directory
  &error("directory_exists", "The directory $outdir already exists. Try again with a different directory name.") if (-e "$outdir" );

########################
# create board directory 
########################
  mkdir $outdir, 0775;

  # we need to chgrp to customgroup value if one exists 
  if ($customgroup{$dept}) {
    system("chgrp $customgroup{$dept} $outdir");  # change group ownership for each dept
  }

  chmod (0770, $outdir);
  system "chmod g+s $outdir";

  # important variables for later
  $dir = $outdir;
  $url = "$baseurl/$dirname";       # base URL for this class.


  chdir $outdir || &error("$_CHDIR","$_COULDNOTCHDIR $outdir");
  # couldn't get unzip to work so am copying instead
  # start of replacement for unzip
  if (!$ta_uid){
    # location of payload data
    $masterbbs = "$BASE/prof/payload/wwwboard";
  } else {
    # location of ta payload data
    $masterbbs = "$BASE/ta/payload/wwwboard";
  }
  if(!$homework) {
    mkdir "$dir/messages", 0755;   # make wwwboard/messages directory
  }

  # Added to try to get classweb as group and sticky bit set
  chmod (0770, $dir);
  if(!$homework) {
    chmod (0770, "$dir/messages");
  }
  system "chmod g+s $dir";
  if(!$homework){
    system "chmod g+s $dir/messages";
    system "cp $masterbbs/data.txt $dir";
  }
  if(!$homework){
    system "cp $masterbbs/wwwboard.pl $dir/wwwboard.pl";
    system "cp $masterbbs/post.pl $dir/post.pl";
    system "cp $masterbbs/wwwboard.inc $dir/wwwboard.inc";
  } else {
    system "cp $homework_dir/wwwboard.homework.pl $dir/wwwboard.pl";
    system "cp $homework_dir/post.homework.pl $dir/post.pl";
    system "cp $homework_dir/homework.php $dir/homework.php";
    system "cp $homework_dir/wwwboard.homework.inc $dir/wwwboard.inc";
  }
  chmod 0770, "$dir/post.pl"; 
  chmod 0770, "$dir/wwwboard.inc"; 

  # end of replacement for unzip

###############################################################

  # make mail.txt file with TA's email address, if found

  $cmd = "select email from profsloc where uid = '$ta_uid'";
  $sth = $dbh->prepare($cmd);
  $sth->execute;
  $emailhash = $sth->fetchrow_hashref;
  $emailaddress = $emailhash->{email};
  if ($emailaddress =~ /^$/) {
    $cmd = "select email from REGta where uid = '$ta_uid' && term='$term' && srs='$srs'"; 
    $sth = $dbh->prepare($cmd);
    $sth->execute;
    $emailhash = $sth->fetchrow_hashref;
    $emailaddress = $emailhash->{email};
  }

  # now write out the mail.txt file
  open (MAILFILE, ">$dir/mail.txt");
  print MAILFILE "$emailaddress";
  close (MAILFILE);
  chmod (0660, "$dir/mail.txt");

####################################################




####################################################
# begin of rest of clonebbs script

  chdir $dir || &error("$_CHDIR","$_COULDNOTCHDIR $dir");
  open (OUT,">$dir/.htaccess")  || &error(".htaccess","$_COULDNOTOPEN $dir/.htaccess");
  print OUT "Options ExecCGI Includes\n"; # do we need Includes anymore?
  print OUT "DirectoryIndex index.php\n";
  close(OUT);
  chmod (0660, "$dir/.htaccess");
  if(!$homework){
    chmod 0660, 'data.txt';
    chmod 0770, 'wwwboard.pl';
  } else {
    chmod 0770, 'wwwboard.homework.pl';
  }

####################################################
# use templates to write out this page

  $FORM{'posturl'} =  "$url/post.pl";
  $FORM{'sposturl'} = $FORM{'posturl'};
  $FORM{'sposturl'} =~ s/^http:/http:/i;
  $FORM{'term'} = $term;
  $FORM{'title'} = $title;
  $FORM{'srs'} = $srs;
  $FORM{'dirname'} = $dirname;
 
  if(!$homework) { 
    &template("index.php","index.php");  
  } else {
    &template("../../homework/index.homework.php","index.php"); 
  } 
  &template("../../auth.inc","auth.inc");  # Authentication control file

  chmod (0770, "$basedir/admin/$dirname.pl");
  
  if($homework == 1) { 
    if(open AUTH, ">>auth.inc"){
      print AUTH "<? \$homework='1';    # Homework=1, Normal=0, default=0\n ?>";
      close AUTH;
    }
  }

  # Add call to update boards table when new board created
  if ($homework == 1) {
    $type = "homework";
  } else {
    $type = "extra";
  }
    
  if(!$ta_uid){
    $site = "class";
  } else {
    $site = "ta";
  }
  
  $auth = "Y";

  $class_dir = $basedir;
  if(!$ta_uid){
    $class_dir =~s/^.+\/([^\/]+)$/$1/;
  } else { 
    $class_dir =~s/^.+\/([^\/]+)\/([^\/]+)$/$1\/$2/;
  }
  $directory = "$term/$class_dir/$dirname/";
  &update_boards($term, $srs, $directory, $auth, $type, $site) ||
    die("update_boards.pm failed.");
  &update_boardnames($term, $srs, $directory, $title, $creator) ||
    die("update_boardnames.pm failed.");

} #write_page


####################################################
# Name: error
# Usage: &error($error, @error_fiedls)
# Update:
# Status: production
# Purpose: print out error message and terminate

sub error {

  ($error,@error_fields) = @_;

  require "$BASE/contenttype.inc"; # enable charset
  # print "Content-type: text/html\n\n";
  print "<html>\n <head>\n  <title>$_ERROR: $error</title>\n </head>\n";
  print " </head>\n <BODY BGCOLOR=#FFFFFF>\n";

  print "   <h1>$_ERROR: $error</h1>\n\n";
  foreach $error_field (@error_fields) {
    print "$error_field\n";
  }

  # added by Artin and mike (boy, I bet their parents are proud! :-)

  # contact of classweb admin
  print "$_BROWSERLEFTARROW\n";
  print "</body></html>\n";
  exit;
} # end of error subroutine


####################################################
# Name: template	
# Usage: &template($infile, $outfile)
# Update:
# Status: production
# Purpose: use input templates combined with $FORM vars to write out file

sub template {
  my($in,$out) = @_;
  open(MODEL,"<$payload/$in");
  open (HOME, ">$outdir/$out") || 
    &error("$out","$_COULDNOTOPEN $outdir/$out"); 

  # this section replaces the #=codes=# with variables from FORM hash
  while (<MODEL>) {
    if ( /\?>/) {
      if ($in eq 'auth.inc') {
          print HOME "\$title='$title';\n";
      }
    }
    s/#=(.*?)=#/$FORM{$1}/g;   # variable between #= and =#
    print HOME;
  }
    
  close(MODEL);
  close(HOME);
  chmod (0660, "$outdir/$out");

} # end of template subroutine

##############################################################################

$link = param('title');
if (param('addlinkname') eq 'yes' && $link ){
  &Place_link($url,$link);     # written by art
}
require "$BASE/contenttype.inc"; # enable charset
# print "Content-type: text/html \n\n";
print "<html>\n <head>\n  <title>$title $_BOARDCREATED_TITLE</title>\n </head>\n";
print " </head>\n <BODY BGCOLOR=#FFFFFF>\n";
  
print "<h2>$title $_BOARDCREATED_HEADING</h2>\n<p>";
print "$_ADDEDBOARDLINK_A <b>$title</b> $_ADDEDBOARDLINK_B<p>";
print "View the <a href = \"$baseurl/$dirname\">$title Board</a>\n<br>";
print "$_RETURNTO <a href = \"$baseurl\">$_CLASSPAGE</a>\n<br>";
print "$_RETURNTO <a href = \"$baseurl/admin?type=discussion\">$_BOARDADMIN</a>\n<br>";
print "</body>\n</html>\n"; 

} # end of addboard subroutine
1
