#!/usr/local/bin/perl 
####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
# Name: automail.pm 
# Purpose: send out announcement of board posting to those in automail.txt 
# Usage: &autonotify($board_name, $url, $author, $subject)

# Update: 4-6-00 mike - error subroutine substituted for wwwboard.pm's and 
#                caused anyone to be able to post. Renamed it to sub merror
# Update: 2-18-00 mike - ready to make production
# Update: 2-16-00 mike trying to incorporate &includephpvars to help get dept
# coursenumber for email header the way bbcheckdb.pl does.
# Update: 2-11-00 mike changed name to autonotify from notify
# Update: 2-10-2000 Yelena The script appears to work with minimal testing only.
#          need to try it in history.  Modified the get_recipients scrpt	
#          to return a string with the list of email addresses instead of an 
#          array.
# Update: 2-4-2000 yelena started this
# Status: writing
# Needs: more testing, including for some History sites.

# variables inherited from wwwboard.pl stub.  eg: 
#   $basedir: /info-data/www-data/00W/women185-4/wwwboard 
#   $baseurl: http://www.sscnet.ucla.edu/00W/women185-4/wwwboard 

#base directory for the class websites
$BASE_DIRECTORY = '$DOCUMENT_ROOT';

$DISCUSSION_MESSAGE_DIRECTORY = 'messages';

#the e-mail address of the person to be contacted in case of an error
$CONTACT_EMAIL = '$EMAIL_OF_CLASSWEB_ADMIN';
# $cc = 1;  # set this if you want cc's to be sent to $CONTACT_EMAIL list

#this file holds text that is appended to each outgoing mail message
$MAIL_POSTSCRIPT_FILE = '$BASE/mail.postscript.txt';

#this is the analogous file for the history department
$H_MAIL_POSTSCRIPT_FILE = '$BASE/mail.history.postscript.txt';

#people to mail when a message has been posted
$MAIL_FILE='automail.txt';

sub autonotify {
# $debug=1;  # 1 turns on debug mode    
  local($msg,*FILE);
  local($contacts); # will come from automail.txt file
  local($dir,$url,$board_name,$boardx,$buf);
  local($author,$subject,$test,$xa,$xb);
  local($history) = 0; #is this a board for a history class?

  $board_name = shift(@_);
  $parse_board = $base_url = $url = $dir = $board = shift(@_);
  $author = shift(@_);
  $subject = shift(@_);
  
# strip off messages/?.php to get board URL
  $base_url =~ s#messages/\d+.php$##;

# need to convert URL into board name to be used in Subject line of email
  $parse_board =~ s#http://$SERVER_NAME/##;
  $parse_board =~ s#/messages/\d+.php$##;
  $parse_board =~ s#/# #;   # replace / with space, first time it occurs only
  ($xa,$xb) = split(/\s/,$parse_board); 
  $xa = uc($xa);
  if ($debug) {
    print STDERR "PHPVARS: \$board_name=$board_name\n";
    print STDERR "PHPVARS: \$url=$url\n";
    print STDERR "PHPVARS: \$base_url=$base_url\n";
    print STDERR "PHPVARS: \$xa=$xa\n";
    print STDERR "PHPVARS: \$xb=$xb\n";
    print STDERR "PHPVARS: \$author=$author\n";
  }

  #load the mail pre- and postscript files
  &load_mail_files();

  if(!$board_name){
    &merror("Couldn't find class name for $board!\n");
    $board_name = $board;
  }

  $contacts = &get_recipients();

  $from = "franks";
  $buf = 'From: $DEFAULT_SENDER\n';
  $buf .= "Subject: Automail: new post on $xa $xb\n\n";

  #poor grammar will not tolerated be :-)
  $buf .= "This is an automated message to notify you that the message \n";
  $buf .= "below has just been posted to your discussion board at\n";
  $buf .= "$base_url\n\n";

  $author = "(not available)" unless $author;
  $subject = "(not available)" unless $subject;

  $buf .= "  Subject: $subject\n";
  $buf .= "  Poster: $author\n";
  $buf .= "  URL: $url\n\n";
  $buf .= $mail_post;

  &send_mail($contacts, $buf);
  &send_cc_mail($CONTACT_EMAIL, $buf, $contacts) if($cc && !$debug);
}


sub send_cc_mail
{
  $_[1] =~ s/http:(.*)/http:$1 ($_[2])/;
  &send_mail($_[0], $_[1]);
}

sub send_mail
{
  #e-mail address of the person to be contacted
  local($ea) = $_[0];
  #the text of the message to send
  local($msg) = $_[1];

  if(!$ea){
    &merror("NULL e-mail address for the following msg:\n$msg");
    return;
  }
  $ea = $CONTACT_EMAIL if($debug);

  #add To: header
  $msg = "To: $ea\n$msg";

  open(MAIL,"|/usr/lib/sendmail -t") || &merror("Couldn't open mail program (!$)\
n");
  print MAIL $msg;
  close(MAIL);
}


sub merror # used to be called error and conflicted with wwwboard.pm's error
{
  $mail_error_text .= $_[0]
}

sub load_mail_files
{
  local(*FILE);
  undef $/;

  $h_mail_post = $mail_post = "";

   if(open(FILE,$H_MAIL_POSTSCRIPT_FILE)){
    $h_mail_post = <FILE>;
  } else {
    if($debug){
      &merror("Couldn't open history mail postscript file because: $!\n");
    }
  }

  if(open(FILE,$MAIL_POSTSCRIPT_FILE)){
    $mail_post = <FILE>;
    close(FILE);
  } else {
    if($debug){
      &merror("Couldn't open mail postscript file because: $!\n");
    }
  }

  $/ = "\n";
}

###########################################
# Name: get_recipients
# Usage: &get_recipients()
# Update: 2/10/2000  Yelena and Mike: will now append emails to sting 
#    $new_contacts instead of creating an array 
# Status: Production
# Purpose: to read automail.txt and construct a list of people to be mailed

sub get_recipients
{
  local(*FILE);
  my $new_contacts;  # or should this be local?

  # we know the file exists or this routine wouldn't have been called.
  # But we need to read it to get addresses of who to auto-notify
  #  assumes addresses are one per line

  if (open(FILE,"$basedir/$MAIL_FILE")){
    while(<FILE>){
      s/\s+//;
      next if(/^$/);
    # print "Adding $_ to $basedir's contact list<br>\n" if($debug);
      if ($new_contacts) {
        $new_contacts = $new_contacts . " , " . $_;
      } else {
        $new_contacts = $_;
      }
    }
    close(FILE);
  } else{
    if($! ne "No such file or directory"){
      print "Open return code--> $!\n";
    }
  }

  return $new_contacts;
}
1
