#!/usr/local/bin/perl

# Name: bbcheckdb.pl
# Usage: bbcheckdb.pl term

# Update: 08-10-00 kenny - removing hardcored vars; adapting for opensource
# Update: 4-5-00 mike - looking for bug 188 around 398 where it dies on xlisted
#                       I think it was missing rest of INSERT INTO statement
# Update: 4-3-00 mike - changed back to franks for email
# Update: 3-13-00 mike - changed from franks to gopher email
# Update: 10-24-1999 Mike Franks - bugfix where nomail.txt was still allowing
#         email to faculty
# Update: 09-27-1999 Adam Kaplan - changes to make sure $msg_count{$board} is
#         initialized in the case that no /messages directory exists
# 09-02-99 yelena changed classweb3 to classweb4 
# 04-13-99 mike added "ignore admin profs: profcode < 3
# 04-07-99 mike added $term to email title and body
# 04-06-99 Bye-bye bb.data.txt. High water marks now kept in database.
# 04-05-99 The script now takes an argument stating which quarter to process.
# 01-14-99 Added check for e-mail address of "none".
#          Now that history classes are in classweb some of the checks
#          needed to be changed (and the key in ctable was made "hist")
# 01-08-99 Fixed bug that caused files like "1.html.old" to be
#          considered a message
# 10-14-98 Changed flow of execution.
# 10-01-98 Now stores board counts in the database (using the boards table)
# 07-01-98 Changed to use 981 - Summer Session
# 06-11-98 Now outputs a different type of html file, one that shows the
#          total number of postings to date for each department's discussion
#          boards; it also shows how many boards each department has.
# 06-10-98 Now outputs a summary html file.
# 05-27-98 Fixed bug introduced by the introduction of an image anchor
#          into the origname field of the saved messages.
# 05-20-98 Removed the log. Exigent error messages are now incorporated
#          into the status e-mail sent to $CONTACT_EMAIL. Also fixed the
#          formating problems with the results table.
# 05-18-98 Added enhanced reporting output
# 05-14-98 Changed message text depending on history/non-history
# 04-05-98 Changed to use 98S
# 01-21-97 Approximate date of inception
# Status: not sure
# Purpose: checks the bbs database


## $BASE is set in Apache Environment with SetEnv and PassEnv in httpd.conf
$BASE = "/home/classweb"; # main classweb programs directory

# if bbcheckdb.pl is run from a cronjob or the command line
# following code is necessary to determine the BASE prog. dir
unless ($BASE) { # If $BASE doesn't exist, use calling path or current directory
  use File::Basename;
  $BASE = dirname($0); # set $BASE to this program's calling path
  unless ($BASE) {
    # for programs called from this directory via telnet command with . on path
    $BASE=".";  # set to current directory if it still doesn't exist
  }
}
require "$BASE/customize.pl";  # important vars set here
require "$BASE/dbvars.pl";     # database vars set here
require("$BASE/dbopen.pl");

#This variable specifies the summary HTML page.
$HTML_SUMMARY = "$DOCUMENT_ROOT$ENTRY/admin/boards.html";

#If this variable is set to 1, then debugging is turned on; if it's
#set to 0, debugging is turned off. if debugging is on then it emails
#$CONTACT_EMAIL instead of faculty and it doesn't write to the data
#file (nor update the database).
$debug = 0;

#if this variable is non-zero then mail to the professors is also sent
#to $CONTACT_EMAIL (unless $debug is set)
$cc = 0;

#the file which contains the highest already used message number.
#the file itself lives in the wwwboard (or equiv.) directory, not in
#the messages directory.
$DISCUSSION_COUNT_FILE = 'data.txt';

#base directory for the class websites
$BASE_DIRECTORY = $DOCUMENT_ROOT;

#base url for class websites
$BASE_URL = "$SERVER_NAME/";

#The file that will hold the e-mail addresses
#to be contacted concerning new messages. These
#addresses, if present, are an exhaustive list
#of the people to be contacted. The "normal"
#address(es) are disregarded. The format of
#the mail file is one e-mail address per line.
$MAIL_FILE = 'mail.txt';

#The presence of this file in the class's directory
#indicates that no one from this class should be
#notified of new messages
$NO_MAIL_FILE = 'nomail.txt';

#if($#ARGV < 0){   # $#ARGV returns subscript of last element of array, since
#                  # arrays usually start with location 0, < 0 should mean array
#                  # is empty 
#	print STDERR "Wrong number of args!!!!\n";
#	exit(1);
#}
#$term = $ARGV[0];

#this file holds text that is appended to each outgoing mail message
$MAIL_POSTSCRIPT_FILE = "$BASE/mail.postscript.txt";

#the e-mail address of the person to be contacted in case of an error
$CONTACT_EMAIL = $EMAIL_OF_CLASSWEB_ADMIN;

#just what it looks like...
$DISCUSSION_MESSAGE_DIRECTORY = 'messages';

# do term looping from here
$gquery = "select term from current2";
$gqh = $dbh->prepare($gquery) or &error("Can't prepare $gquery: $dbh->errstr\n");
$rv = $gqh->execute or &error("can't execute the query: $gqh->errstr\n");
while(($term) = $gqh->fetchrow_array) {

  #this variable is used to hold all the status messages that should
  #be included in the status e-mail
  $mail_text = "State of affairs when bbcheck.pl was run for $term on ";
  $mail_text .= localtime(time()) . ":\n";
  
  #actual error messages included in the status message
  $mail_error_text = "";
  
  #local variables
  local(%prev_count, @new_msgs, %been_emailed, %new_count, @boards);
  local($profemail, $srs, $done_indirect_urls) = ("","",0);
  
  #used to keep track of which boards have already been processed.
  local(%visited);
  
  #these variables are used in the accounting process
  local(%ctable,%totals,%btotals);
  
  #grand totals (boards and postings, respectively)...
  local(%gbtotals,%gptotals);
  
  #load the mail pre- and postscript files
  &load_mail_files();
  
  #load the counts for the bulletin boards
  &load_prev_count();
  
  #initialize the hash tables related to accounting
  &init_hash_tables();
  
  #extract all URLs in the REGurl table
  #$cmd = "select a.srs,url from REGurl a, REGclasses b where a.term='$term' && a.term=b.term && a.srs=b.srs && b.status <>'Cancelled'";
  $cmd = "select srs,url from REGclasses where term='$term' && status <>'Cancelled'";
  $sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
  $sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
  LOOP:
  while(($srs,$url) = $sth->fetchrow_array){
  	next unless($url =~ /^$BASE_URL/i);
    if($url =~ /(.*)\/(.*)\.htm/){
  		$url = "$1/";
  	}
  	$url =~ s/^$BASE_URL//i;
  	$url =~ s#^(/)+##;
  	$dir = $url;
  	print "dir --> $dir\n" if($debug);
  	unless(-e "$BASE_DIRECTORY/$dir"){
  		print STDERR "$dir does not exist!\n" if($debug);
  		&error("$dir doesn't exist!\n");
  		next;
  	}
  
  	if(defined($visited{$dir})){
  		next;
  	} else {
  		$visited{$dir} = 1;
  	}
  
  	print "url--> $url\n" if($debug);
  	foreach(keys(%ctable)){
  		if($url =~ /^$_/){
  			$btotals{$_}++;
  			last;
  		}
  	}
  
  	@boards = &get_discussion_boards($dir);
  
  	print "working on $dir (@boards)\n" if ($debug);
  
  	foreach(keys(%ctable)){
  		#cache the key for later use
  		$key = $_;
  		if($dir =~ /$key/){
  			$gbtotals{$key} += @boards;
  			last;
  		}
  	}
  
  	$class_name = &get_class_name($srs);
  	@profemail = @profemail2 = &get_prof_email($srs);
     next if(@profemail && $profemail[0] eq "none"); ## none indicates no e-mail address
  	print "class --> $class_name\tprof --> @profemail\n" if($debug);
  
  	foreach $b (@boards){
	  $b="$ENTRY/$b";
      #Does this board have a file indicating that they shouldn't
      #be mailed? If so, skip them. Mike moved this check here  10-24-99 instead
      # of inside get_recipients call to ignore function. All that had been
      # happening was that mail.txt was being ignored, not faculty
      next if (-e "$BASE_DIRECTORY/$b/$NO_MAIL_FILE");
  
  		#if this is a discussion board run by a TA then this routine
  		#will return the ta's e-mail address; otherwise, it'll return ()
  		@ta_email = &get_ta_email($b);
  		@profemail = @ta_email if(@ta_email);
  
  		print "board --> $b\n" if($debug);
  		&get_exact_message_count($b);
  		@eadds = &get_recipients($b);
  		if($#profemail < 0 && $#eadds < 0){
  			&error("No contact info for $b\n");
  			print "No contact info for $b\n" if($debug);
  			next;
  		}
  
  		@profemail = () if($#eadds >= 0);
  
  		$gptotals{$key} = 0 unless(defined($gptotals{$key}));
  		$gptotals{$key} += &get_total_messages($b);	
  		if((@new_msgs = &get_new_messages($b))){
  			print "$b has new messages (@new_msgs)\n" if($debug);
  			print "contacts for $b --> " . join(',', @eadds, @profemail) . "\n" if($debug);
  			&notify(join(',', @eadds, @profemail), $class_name, $b, @new_msgs);
  		}
  
  		&write_out_msg_count($srs, $b);
  		@profemail = @profemail2;
  	}
  
  	$profemail = $class_name = "";
  }
  
  if(!$done_indirect_urls){
  	$done_indirect_urls = 1;
  	$cmd = "select srs,url from indirect_url where term='$term'";
  	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
  	$sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
  	goto LOOP;
  }
  
  unless($debug){
  	open(FILE,">$HTML_SUMMARY");
  	print FILE "<html>\n<head>\n<title>Summary of $term Discussion Board Activity</title>\n";
  	print FILE "</head><body bgcolor='#FFFFFF'>\n";
  	print FILE "<table border>\n";
  	print FILE "<caption>Numbers as of " . scalar(localtime(time())) . "</caption>\n";
  	print FILE "<tr><th>Department</th><th>Total # of Boards</th><th>Total # of Postings</th></tr>\n";
  	foreach(sort(keys(%gbtotals))){
  		print FILE "<tr align='center'><td>".$ctable{$_}."</td><td>".$gbtotals{$_}."</td><td>".$gptotals{$_}."</td></tr>\n";
  	}
  	print FILE "</table>\n</body></html>";
  	close(FILE);
  }
  
  &send_status_mail();
}
exit(0);


###########################################
# Name: send_status_mail
# Usage: &send_status_mail
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: prints/sends email of board/posting stats

sub send_status_mail
{
	local(@lines, $buffer);
	#the grand total number of postings and boards, respectively
	local($gtp, $gtb);
	#the grand total number of boards posted to
	local($gtpb);
	#used in formatting
	local($l1, $l2, $l3, $len1, $len2, $len3, $len4);
	local($a, $b, $c, $d);

	@lines = split(/\n/, $mail_text);
	$buffer = shift(@lines);
	$buffer .= "\n\n";
	@lines = sort(@lines);
	foreach(@lines){
		$buffer .= "$_\n";		
	}	

	$buffer = "Subject: Summary of $term bbcheckdb.pl\n\n$buffer";
	$buffer .= "\n" . ' 'x22 . "Breakdown by Department\n";
	if($debug){
		print "\n" . ' 'x22 . "Breakdown by Department\n";
		print "   ----------            -------------   -----------   -----------------\n";
		print '-'x70 . "\n\n";
		print "   Department            # of Postings   # of Boards   Total # of Boards\n";
	}

	$buffer .= '-'x70 . "\n\n";
	$buffer .= "   Department                    # of Postings   # of Boards   Total # of Boards\n";
	$buffer .= "   ----------                    -------------   -----------   -----------------\n";

	$gtp = 0;
	$gtb = 0;
	$gtpb = 0;

	foreach(sort(keys(%totals))){
		$a = $ctable{$_};
		$b = $gptotals{$_};
		$c = $ptotals{$_};
		$d = $gbtotals{$_};
		$len1 = length($a);
		$len2 = length($b);
		$len3 = length($c);
		$len4 = length($d);

		$l1 = 35 - $len1 + $len2 - (int($len2/2)+1);
		$l2 = 50 - ($len1 + $l1 + $len2) + $len3 - (int($len3/2)+1);
		$l3 = 67 - ($len1 + $l1 + $l2 + $len2 + $len3) + $len4 - (int($len4/2)+1);

		$buffer .= sprintf "%s     %".$l1."s %" . $l2 . "s %" . $l3 . "s\n", $a, $b, $c, $d;
		if($debug){
		 printf "%s  %".$l1."s %" . $l2 . "s %" . $l3 . "s\n", $a, $b, $c, $d;
		}

		$gtp += $b;
		$gtb += $d;
		$gtpb += $c;
	}

	$buffer .= "\n" . "\n\t\t\t*******Totals*******\n" . '-'x70; 
	$buffer .= "\n$gtp total new posting(s) on $gtpb/$gtb boards.\n";
	if($mail_error_text){
		$buffer .= "\n\t\t\t******Errors*******\n" . '-'x70 . "\n";
		$buffer .= $mail_error_text;
	}

	&send_mail($CONTACT_EMAIL, $buffer);
}

###########################################
# Name: get_auth
# Usage: $yesorno = &get_auth($board)
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: tells whether or not board is authenticated

sub get_auth
{
	local($board) = $_[0];
	$board =~ s/^$ENTRY\///;
	local(*FILE);
	
	if(open(FILE, "$BASE_DIRECTORY$ENTRY/$board/auth.inc")){
		while(<FILE>){
			if(/\$authenticated='(\d)'/){
				if($1 == 1){
					return 'Y';
				} else {
					return 'N';
				}
			}
		}
	} else {
		&error("Couldn't open $board" . "'s auth.inc\n");
	}

	return 'N';
}


###########################################
# Name: write_out_msg_count
# Usage: &write_out_msg_count($srs, $board);
# Update:10/11/99 Yelena removing srs from Update query.  This way crosslisted
#          classes which share a board will all get updated instead of just 
#          the first one. 
#        7/12/99 Adam Kaplan - commenting
# Status: Production 
# Purpose: updates boards table in database

sub write_out_msg_count
{
	local($srs, $board) = @_;
	$board =~ s/^$ENTRY\///;
	local($cmd, $sth, @row, $cnt);

	return if($debug);
	$been_emailed{$board} = 1;	
	if(defined($been_emailed{$board})){
		$cnt = $new_count{$board};
	} else {
		$cnt = $prev_count{$board};
	}

	$auth = &get_auth($board);

	if(defined($been_emailed{$board})){
		print "Updating $board [$cnt]\n" if($debug);
		$cmd = "select * from boards where term='$term' && srs='$srs'";
   	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   	$sth->execute || &maildie ("Can't execute query: ". $sth->errstr ."\n");
   	@row = $sth->fetchrow_array;

		#the board isn't in the database, so we insert it
		if($#row < 0){
		  $cmd = "INSERT INTO boards (term,srs,directory,count,auth,hwm) ";
      		  $cmd .= "VALUES('$term','$srs','$board',$msg_count{$board},'$auth',$cnt)";
		} else {
      		  $cmd = "UPDATE boards set count=$msg_count{$board}, hwm=$cnt where term = '$term' && directory='$board'";
		}
   	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   	$sth->execute || &maildie ("Can't execute query [$cmd] for line 398 $term $srs $board: ". $sth->errstr ."\n");
	} else {
		print "$board is new!\n";
		$cmd = "select * from boards where term='$term' && srs='$srs'";
   	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   	$sth->execute || &maildie ("Can't execute query: ". $sth->errstr ."\n");
   	@row = $sth->fetchrow_array;

		#the board isn't in the database, so we insert it with zero postings.
		if($#row < 0){
			$board =~ s/^$BASE_DIRECTORY\///;
			$cmd = "INSERT INTO boards (term,srs,directory,count,auth,hwm) ";
      $cmd .= "VALUES ('$term','$srs','$board',0,'$auth',0)";
			$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
			$sth->execute || &maildie ("Can't execute query: ". $sth->errstr ."\n");
		}
	}
}


###########################################
# Name: load_prev_count
# Usage: &load_prev_count
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: prints old high water mark for each directory

sub load_prev_count
{
	local($dir, $hwm);

	$cmd = "select directory,hwm from boards where term='$term'";
 	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
	$sth->execute || &maildie ("Can't execute query: ". $sth->errstr ."\n");

 	while((($dir, $hwm) = $sth->fetchrow_array)){
		print "old high water mark for $dir --> $hwm\n" if($debug);
		$prev_count{$dir} = $hwm;
	}
}



###########################################
# Name: get_exact_message_count
# Usage: &get_exact_message_count($board)
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: prints exact message count for board

sub get_exact_message_count
{
	local($board) = $_[0];
	$board =~ s/^$ENTRY\///;
	local(*DIR, $dir, $count);

	$dir = "$BASE_DIRECTORY$ENTRY/$board".$DISCUSSION_MESSAGE_DIRECTORY;
	if(opendir(DIR, $dir)){
      $count = grep(-f "$dir/$_" && /^\d+.php$/, readdir(DIR));
      rewinddir(DIR);
      $count = grep(-f "$dir/$_" && /^\d+.html$/, readdir(DIR)) unless($count);
      closedir(DIR);
		$msg_count{$board} = $count;
	} else {
		&error("Couldn't opendir $dir: $!\n");
                $msg_count{$board} = 0;  # added by Adam Kaplan 9-27-1999
	}

	print "exact msg count for $board --> $msg_count{$board}\n" if($debug);
}



###########################################
# Name: get_discussion_boards
# Usage: @hits = get_discussion_boards($dir)
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: finds all discussion boards in a directory

sub get_discussion_boards
{
	local($dir) = $_[0];
	local($file);
	local(@hits);

	@hits = &make_uniq(0, &find_file("$BASE_DIRECTORY/$dir", $DISCUSSION_COUNT_FILE));
	foreach(@hits){
		$_ =~ s/^$BASE_DIRECTORY$ENTRY\///;
		$_ .= '/' unless(/\/$/);
	}

	return @hits;
}


###########################################
# Name: find_file
# Usage: @hits = find_file($dir, $file)
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: finds a file in a directory
# Needs: Change this to use the FILE::Find package.

sub find_file
{
	local($dir) = $_[0];
	local($file) = $_[1];
	local(*PIPE,@hits);

	open(PIPE,"find $dir -name $file -print|");
	@hits = <PIPE>;
	close(PIPE);

	foreach(@hits){
		chop;
		s/\/$file$//;
	}

	return @hits;
}



###########################################
# Name: get_total_messages
# Usage: $count = &get_total_messages($board)
# Update: 7/12/99 Adam Kaplan - commenting
# Status: not sure
# Purpose: gets total message count for a board

sub get_total_messages 
{
	local(*FILE, $file);
	local($board) = $_[0];
	$board =~ s/^$ENTRY\///;
	local($count) = 0;
	$file = "$BASE_DIRECTORY$ENTRY/$board/data.txt";
		print "checking $BASE_DIRECTORY$ENTRY/$board/data.txt for prev_count...\n" if($debug);

	if(open(FILE, $file)){ 
		$count = <FILE>;
		if($count){
			chomp($count);
		} else {
			$count = 0;
		}
		close(FILE);
	} else {
		&error("Couldn't open $file in get_total_messages(): $!\n");
	}

	if(!defined($prev_count{$board})){
		print "$board has no prev_count!!\n" if($debug);
		$prev_count{$board} = 0;
	}
	print "$board has $count messages [prev: $prev_count{$board}]\n" if($debug);

	$new_count{$board} = $count;
	return $count;
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

sub get_new_messages
{
	local(*FILE, *DIR, @msgs, $dir, $m);
	local($board) = $_[0];
	$board =~ s/^$ENTRY\///;
	#the number of the first new message
	local($n,@hits,@keys,$count);
	local(@new_msgs);

	$count = 0;	
	$dir = "$BASE_DIRECTORY$ENTRY/$board/$DISCUSSION_MESSAGE_DIRECTORY";

	if(($n = &has_changed($board)) >= 0){
		if(opendir(DIR,"$dir")){ 
			@msgs = grep(-f "$dir/$_",readdir(DIR));
			closedir(DIR);
		} else {
			&error("Couldn't opendir $dir in hnm(): $!\n");
		}

		foreach $m(@msgs){
			$count++;
			if(($m =~ /(\d+).html$/ && $1 > $n) || ($m =~ /(\d+).php$/ && $1 > $n)){
				push(@new_msgs, $m);
			}
		}
	}

	return @new_msgs;
}

###########################################
# Name: has_changed
# Usage:
# Update:
# Status: production
# Purpose:

sub has_changed
{
	local($board) = $_[0];
	$board =~ s/^$ENTRY\///;
	#Is this a new directory?
	if(!defined($prev_count{$board})){
		print "new board ---> $board\n" if($debug);
		$prev_count{$board} = 0;
	}
	
	if(defined($new_count{$board}) && $new_count{$board} > $prev_count{$board}){
   	print "high water mark [$board] --> $new_count{$board}\n" if($debug);
		return $prev_count{$board};
	} else {
		$new_count{$board} = $prev_count{$board};
   	print "msg count for $board --> $new_count{$board}\n" if($debug);
		return -1;
	}
}

###########################################
# Name: get_class_name
# Usage:
# Update:
# Status: production
# Purpose:

sub get_class_name
{
	local($srs) = $_[0];
	local($name, $cmd, $sth, @qclass);

	$cmd = "select dept, coursenum from REGclasses where term='$term' && srs='$srs'";
   $sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   $sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   @qclass = $sth->fetchrow_array;
	if($qclass[0] || $qclass[1]){
		$name = "$qclass[0] $qclass[1]";
	} else {
		$name = "Unknown";
	}

	print "class $srs is named $name\n" if($debug);
	return $name
}

###########################################
# Name: notify					
# Usage:
# Update:
# Status: Production
# Purpose:

sub notify
{
	local(@msgs,$msg,*FILE);
	local(@contacts,$contacts,$from);
	local($dir,$url,@keys,$board_name,$buf);
	local($author,$subject,$time,$test);
	local($history) = 0; #is this a board for a history class?

	$contacts = shift(@_);
	$board_name = shift(@_);
	$url = $dir = $board = shift(@_);
	@msgs = @_;	

	$url =~ s/$BASE_DIRECTORY//;
	$url =~ s#//#/#g;
	$base_url = "$BASE_URL/$board/";
	$board =~ s/^$ENTRY\///;
	$url = "$base_url" . $DISCUSSION_MESSAGE_DIRECTORY;
	$test = $board;
	$test =~ s/$term//;
	$test =~ s/^(\/)+//;

	foreach(keys(%ctable)){
		if($test =~ /^$_/){
			$totals{$_} += scalar(@msgs);
			$ptotals{$_}++;
			last;
		}
	}

	if(!$board_name){
		&error("Couldn't find class name for $board!\n");
		$board_name = $board;
	}

	$from = "Postmaster";
	$buf = "From: $DEFAULT_SENDER\n";

	$buf .= "Subject: New postings to $board_name board\n\n";

	$base_url =~ s#/+#/#g;
	#poor grammar will not tolerated be :-)
	if($#msgs == 0){
		$buf .= "This is an automated message to notify you that there has been \n";
		$buf .= "1 recent posting to your $board_name discussion board at\n";
		$buf .= "http://$base_url\n\n";
	} else {
		$buf .= "This is an automated message to notify you that there have been \n";
		$buf .= ($#msgs + 1) . " recent postings to your $board_name discussion ";
		$buf .= "board at\nhttp://$base_url\n\n";
	}

	foreach $msg(@msgs){
		next if($msg =~ /^$/);
		($author, $subject, $time) = &get_message_info($board, $msg);
		$author = "(not available)" unless $author;
		$subject = "(not available)" unless $subject;
		$subject =~ s/\&quot\;/"/g;
		$subject =~ s/\&amp\;/&/g;

		$time = "(not available)" unless $time;
	
		$buf .= "Subject: $subject\n";
		$buf .= "  Poster: $author\n";
		$buf .= "  Time: $time\n";
		$url =~ s#/+#/#g;
		$buf .= "  URL: http://$url/$msg\n\n";
	}

	if($history){
		$buf .= $h_mail_post;
	} else {
		$buf .= $mail_post;
	}
	
	&send_mail($contacts, $buf);
	&send_cc_mail($CONTACT_EMAIL, $buf, $contacts) if($cc && !$debug);	
	&log_mail_message($from, $contacts, $board, scalar(@msgs));
   print "setting been_emailed{$board} = 1\n" if($debug);
	$been_emailed{$board} = 1 unless($debug);
}

###########################################
# Name: log_mail_message
# Usage:
# Update:
# Status: Production
# Purpose:

sub log_mail_message
{
	local($from, $address, $dir, $num_msgs) = @_;
	$dir =~ s/^$DOCUMENT_ROOT\///;
	$mail_text .= "$from mailed $address for $dir($num_msgs posts)\n";
}

###########################################
# Name: send_cc_mail
# Usage:
# Update:
# Status: Production
# Purpose:

sub send_cc_mail
{
	$_[1] =~ s/http:(.*)/http:$1 ($_[2])/;
	&send_mail($_[0], $_[1]);
}

###########################################
# Name: send_mail
# Usage:
# Update:
# Status: Production
# Purpose:

sub send_mail
{
	#e-mail address of the person to be contacted
	local($ea) = $_[0];
	#the text of the message to send
	local($msg) = $_[1];

	if(!$ea){
		&error("NULL e-mail address for the following msg:\n$msg");
		return;
	}
	
	$ea = $CONTACT_EMAIL if($debug);

	#add To: header
	$msg = "To: $ea\n$msg";	

	open(MAIL,"|$SEND_MAIL_CMD") || &error("Couldn't open mail program (!$)\n");
	print MAIL $msg;
	close(MAIL);
}

###########################################
# Name: get_message_info
# Usage:
# Update:
# Status: Production
# Purpose:

sub get_message_info
{
	local($file) = "$BASE_DIRECTORY$ENTRY/$_[0]/$DISCUSSION_MESSAGE_DIRECTORY/$_[1]";
	local($poster,$subject,$time,$tmp);

	if(open(FILE,$file)){
		while(<FILE>){
			if(/<title>(.*)<\/title>/i){
				$subject = $1;
			} elsif(/name="origname"\s* value="(.*)"/i){
				$poster = $1;
				$poster =~ s/\s+$//;
			} elsif(/name="origname"\s* value='(.*)'/i){
				$tmp = $1;
				$tmp =~ /<img src="(.*)">\s*(.*)$/i;
				$poster = $2;
				$poster = $tmp unless($poster);
				$poster =~ s/\s+$//;
			}
		}
		close(FILE);
	} else {
		&error("Couldn't open $file in get_message_info: $!\n");
		return "", "", "";
	}

	$time = localtime((stat($file))[9]);
	return $poster, $subject, $time;
}

###########################################
# Name: ignore
# Usage:
# Update:
# Status: Production
# Purpose:

sub ignore
{
	#Does this class have a file indicating that they shouldn't
	#be mailed?
	return (-e "$_[0]/$NO_MAIL_FILE");
}

###########################################
# Name: get_prof_email
# Usage:
# Update:
# Status: Production
# Purpose:
#Gets a list of the e-mail address(es) for the professor(s) of the
#class in question. It first consults profsloc; if it finds something
#it uses that, otherwise it continues on to the REGfaculty table.
sub get_prof_email
{
	local($srs) = $_[0];
	local(@contact_list, $email, @profuids);
	local($cmd, $sth, %hash, $uid);

# $cmd = "select uid from REGprof where term='$term' && srs='$srs'";
# Ignore administrator profs - mike 4-13-99
	$cmd = "select uid from REGprof where term='$term' && srs='$srs' && profcode < 3";
   $sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   $sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   while($uid = ($sth->fetchrow_array)[0]){
		print "profuid for class $srs --> $uid\n" if($debug);
		push(@profuids, $uid);
	}

	foreach $uid (@profuids){
		$cmd = "select email from profsloc where uid='$uid'";
   	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   	$sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   	$email = ($sth->fetchrow_array)[0];
		if($email){
			print "Setting email for prof $uid to $email\n" if($debug);
			$hash{$uid} = $email;
		} elsif($debug){
			print "$uid has NULL e-mail address in profsloc...\n";
		}
	}

	foreach $uid(@profuids){
		next if(defined($hash{$uid}));
		$cmd = "select email from REGfaculty where uid='$uid'";
   	$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   	$sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   	$email = ($sth->fetchrow_array)[0];
		if($email){
			print "setting email for prof $uid (from REGfaculty table) to $email\n" if($debug);
			$hash{$uid} = $email;
		}
	}

	foreach $key(keys(%hash)){
		push(@contact_list, $hash{$key});
	}

	return &make_uniq(1, @contact_list);
}

###########################################
# Name: make_uniq
# Usage:
# Update:
# Status: Production
# Purpose:

sub make_uniq
{
	local(%hash);
	local($ignore_case) = shift(@_);

	foreach(@_){
		$_[0] =~ tr/A-Z/a-z/ if($ignore_case);
		$hash{$_} = 1;
	}

	return keys %hash;
}

###########################################
# Name: get_recipients
# Usage:
# Update:
# Status: Production
# Purpose:

sub get_recipients
{
	local($dir) = "$BASE_DIRECTORY/$_[0]";
	local(*FILE,@contact_list);
	
	@contact_list = ();

	#Does this class have a file indicating that they shouldn't
	#be mailed? If it's zero length, no one should be e-mailed;
	#otherwise, everyone *except* those listed in the file should
	#be mailed. The second part isn't currently implemented.
	return () if (&ignore($dir));

	#Does this class have a file listing other people to be
	#contacted in the case of new messages?
	if(open(FILE,"$dir/$MAIL_FILE")){
		print "$dir has a $MAIL_FILE mail file ...\n" if($debug);
		while(<FILE>){
			s/\s+//;
			next if(/^$/);
			print "Adding $_ to $dir" . "'s contact list\n" if($debug);
			push(@contact_list, $_);
		}
		close(FILE);
	} else{
		if($! ne "No such file or directory"){
			print "Open return code--> $!\n";
		}
	}

	return @contact_list;
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

sub get_url_list
{
	local(%hash, @d, $url);
	local($cmd, $sth);

	#extract all URLs in the REGurl table
	#$cmd = "select srs,url from REGurl where term='$term'";
	$cmd = "select srs,url from REGclasses where term='$term'";
   $sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   $sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   while(($srs,$url) = $sth->fetchrow_array){
		next unless($url =~ /$BASE_URL/i);
		$url =~ s/$BASE_URL//i;
		$hash{$url} = $srs;
		$url =~ s#^(/)+##;

		print "url--> $url\n" if($debug);
		foreach(keys(%ctable)){
			if($url =~ /^$_/){
				$btotals{$_}++;
				last;
			}
		}
	}

	#now extract any in the indirect_url table
	$cmd = "select url from indirect_url where term='$term'";
   $sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
   $sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
   while(($srs,$url) = $sth->fetchrow_array){
		next unless($url =~ /$BASE_URL/i);
		$url =~ s/$BASE_URL//i;
		$hash{$url} = 1;
		$url =~ s#^(/)+##;

		print "indirect_url--> $url\n" if($debug);
		foreach(keys(%ctable)){
			if($url =~ /^$_/){
				$btotals{$_}++;
				last;
			}
		}
	}

	@d = sort(keys(%hash));
	&error("Couldn't find any directories!\n") unless(@d > 0);
	return @d;
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

sub error 
{
	$mail_error_text .= $_[0]
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

sub fatal_error
{
	&error($_[0]);
	print STDERR "\n\t\tFatal Error!\n";
	print STDERR "Error --> $mail_error_text\n";
	exit(1);
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

sub load_mail_files
{
	local(*FILE);
	undef $/;

	$h_mail_post = $mail_post = "";

   if(open(FILE,$H_MAIL_POSTSCRIPT_FILE)){
		$h_mail_post = <FILE>;
	} else {
		if($debug){
			&error("Couldn't open history mail postscript file because: $!\n");
		}
	}
		
	if(open(FILE,$MAIL_POSTSCRIPT_FILE)){
		$mail_post = <FILE>;
		close(FILE);
	} else {
		if($debug){
			&error("Couldn't open mail postscript file because: $!\n");
		}
	}

	$/ = "\n";
}

###########################################
# Name:
# Usage:
# Update:
# Status: NEEDS to be switched to database queries!!!
# Purpose:

sub init_hash_tables
{
  #hash table used to translate directories to department
  $gsquery = "select dirname,lname from REGdept";
  $gsqh = $dbh->prepare($gsquery) or &error("Can't prepare $gsquery: $dbh->errstr\n");
  $rv = $gsqh->execute or &error("can't execute the query: $gsqh->errstr\n");
  while(($dirname,$lname) = $gsqh->fetchrow_array) {
    $ctable{$dirname} = "$lname";
    $totals{$dirname} = 0;
    $ptotals{$dirname} = 0;
    $btotals{$dirname} = 0;
    $gbtotals{$dirname} = 0;
    $gptotals{$dirname} = 0;
    print $ctable{$dirname}."\n" if($debug);
  }
#it is very import that "ge" come after "geog" in REGdept
#	$ctable{'aero'} = 'Aerospace Studies';
#	$ctable{'afroam'} = 'Afro American Studies';
#	$ctable{'amind'} = 'American Indian Studies';
#	$ctable{'anthro'} = 'Anthropology';
#	$ctable{'asian'} = 'Asian American Studies';
#	$ctable{'chicano'} = 'Chicano Studies';
#	$ctable{'comm'} = 'Communication Studies';
#	$ctable{'econ'} = 'Economics';
#	$ctable{'euro'} = 'European Studies';
#	$ctable{'geog'} = 'Geography';
#	$ctable{'ge'} = 'General Education Clusters';
#	$ctable{'hist'} = 'History';
#	$ctable{'hnrs'} = 'Honors Collegium';
#	$ctable{'ids'} = 'International Development Studies';
#	$ctable{'latam'} = 'Latin American Studies';
#	$ctable{'milsci'} = 'Military Science';
#	$ctable{'naval'} = 'Naval Science';
#	$ctable{'polisci'} = 'Political Science';
#	$ctable{'soc'} = 'Sociology';
#	$ctable{'speech'} = 'Speech';
#  	$ctable{'women'} = "Women's Studies";
#
#
#	#total number of postings per department
#	$totals{'aero'} = 0;
#	$totals{'afroam'} = 0;
#	$totals{'amind'} = 0;
#	$totals{'anthro'} = 0;
#	$totals{'asian'} = 0;
#	$totals{'chicano'} = 0;
#	$totals{'comm'} = 0;
#	$totals{'econ'} = 0;
#	$totals{'euro'} = 0;
#	$totals{'geog'} = 0;
#	$totals{'ge'} = 0;
#	$totals{'hist'} = 0;
#	$totals{'hnrs'} = 0;
#	$totals{'ids'} = 0;
#	$totals{'latam'} = 0; 
#	$totals{'milsci'} = 0;
#	$totals{'naval'} = 0;
#	$totals{'polisci'} = 0;
#	$totals{'soc'} = 0;
#	$totals{'speech'} = 0;
#  	$totals{'women'} = 0;
#
#	#number of boards posted to per department
#	$ptotals{'aero'} = 0;
#	$ptotals{'afroam'} = 0;
#	$ptotals{'amind'} = 0;
#	$ptotals{'anthro'} = 0;
#	$ptotals{'asian'} = 0;
#	$ptotals{'chicano'} = 0;
#	$ptotals{'comm'} = 0;
#	$ptotals{'econ'} = 0;
#	$ptotals{'euro'} = 0;
#	$ptotals{'geog'} = 0;
#	$ptotals{'ge'} = 0;
#	$ptotals{'hist'} = 0;
#	$ptotals{'hnrs'} = 0;
#	$ptotals{'ids'} = 0;
#	$ptotals{'latam'} = 0; 
#	$ptotals{'milsci'} = 0;
#	$ptotals{'naval'} = 0;
#	$ptotals{'polisci'} = 0;
#	$ptotals{'soc'} = 0;
#	$ptotals{'speech'} = 0;
#  	$ptotals{'women'} = 0; 
#
#	#total number of boards per department
#	$btotals{'aero'} = 0;
#	$btotals{'afroam'} = 0;
#	$btotals{'amind'} = 0;
#	$btotals{'anthro'} = 0;
#	$btotals{'asian'} = 0;
#	$btotals{'chicano'} = 0;
#	$btotals{'comm'} = 0;
#	$btotals{'econ'} = 0;
#	$btotals{'euro'} = 0;
#	$btotals{'geog'} = 0;
#	$btotals{'ge'} = 0;
#	$btotals{'hist'} = 0;
#	$btotals{'hnrs'} = 0;
#	$btotals{'ids'} = 0;
#	$btotals{'latam'} = 0; 
#	$btotals{'milsci'} = 0;
#	$btotals{'naval'} = 0;
#	$btotals{'polisci'} = 0;
#	$btotals{'soc'} = 0;
#	$btotals{'speech'} = 0;
#  	$btotals{'women'} = 0; 
#
#	$gbtotals{'aero'} = 0;
#	$gbtotals{'afroam'} = 0;
#	$gbtotals{'amind'} = 0;
#	$gbtotals{'anthro'} = 0;
#	$gbtotals{'asian'} = 0;
#	$gbtotals{'chicano'} = 0;
#	$gbtotals{'comm'} = 0;
#	$gbtotals{'econ'} = 0;
#	$gbtotals{'euro'} = 0;
#	$gbtotals{'geog'} = 0;
#	$gbtotals{'ge'} = 0;
#	$gbtotals{'hist'} = 0;
#	$gbtotals{'hnrs'} = 0;
#	$gbtotals{'ids'} = 0;
#	$gbtotals{'latam'} = 0; 
#	$gbtotals{'milsci'} = 0;
#	$gbtotals{'naval'} = 0;
#	$gbtotals{'polisci'} = 0;
#	$gbtotals{'soc'} = 0;
#	$gbtotals{'speech'} = 0;
#  	$gbtotals{'women'} = 0;
#
#	$gptotals{'aero'} = 0;
#	$gptotals{'afroam'} = 0;
#	$gptotals{'amind'} = 0;
#	$gptotals{'anthro'} = 0;
#	$gptotals{'asian'} = 0;
#	$gptotals{'chicano'} = 0;
#	$gptotals{'comm'} = 0;
#	$gptotals{'econ'} = 0;
#	$gptotals{'euro'} = 0;
#	$gptotals{'geog'} = 0;
#	$gbtotals{'ge'} = 0;
#	$gptotals{'hist'} = 0;
#	$gptotals{'hnrs'} = 0;
#	$gptotals{'ids'} = 0;
#	$gptotals{'latam'} = 0; 
#	$gptotals{'milsci'} = 0;
#	$gptotals{'naval'} = 0;
#	$gptotals{'polisci'} = 0;
#	$gptotals{'soc'} = 0;
#	$gptotals{'speech'} = 0;
#  	$gptotals{'women'} = 0;
}

###########################################
# Name:
# Usage:
# Update:
# Status:
# Purpose:

###TODO: Should this return an array or a scalar?
# Glen: in the Open Source release, this routine seems unsupported!
sub get_ta_email
{
	local($dir) = "$BASE_DIRECTORY/$_[0]";
	local($srs) = "";
	local($uid) = "";
	local($email) = "";
	local($cmd, $sth);

	print "get_ta_email(dir) --> $dir\n" if ($debug);

	if(-e "$dir/../index.php"){
		if(open(FILE,"$dir/../index.php")){
			while(<FILE>){
				if(/\$ta_uid="(.*)";/){
					$uid = $1;
				} elsif(/\$srs="(.*)";/){
					$srs = $1;
				}
			}
		} else {
			print "NO DICE for $dir/../index.php\n" if ($debug);
		}
	} else {
		print "$dir/../index.php doesn't exist!\n" if($debug);
	}

	if($srs && $uid){
		print "get_ta_email found an srs --> $srs\n" if($debug);
		$cmd = "select dept,coursenum,secidx from REGclasses where srs='$srs' && term='$term'";
		$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
		$sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
		($dept, $cn, $sec) = $sth->fetchrow_array;
		if(!$dept || !$cn || !$sec){
			print "bad result from lookup on srs $srs\n" if($debug);
			return ();
		}

		print "Looking for email for dept=$dept,cn=$cn,sec=$sec,uid=$uid,srs=$srs,term=$term\n" if($debug);
      ### TODO: If the query has "&& term='$term'" it doesn't work! Why??
		$cmd = "select email from REGta where dept='$dept' && coursenum='$cn' && relsecidx=$sec && uid='$uid'";
		$sth = $dbh->prepare($cmd) || &maildie ("Can't prepare $cmd: $dbh->errstr\n");
		$sth->execute || &maildie ("Can't execute query: $sth->errstr\n");
		$email = ($sth->fetchrow_array)[0];
		if($email){
			print "email for TA of class $srs is $email\n if($debug)";
		} else {
			print "email for TA of class $srs is NULL\n if($debug)";
		}
		return $email;
	} else {
		print "No srs for $dir in get_ta_email\n" if($debug);
		return ();
	}
}

###########################################
# Name: maildie
# Usage: &maildie($msg)
# Update: 7/20/99 Adam Kaplan - created
# Status: testing
# Purpose: die and send email to a contact

sub maildie {

  my($msg) = @_;
  my $contact = $EMAIL_OF_CLASSWEB_ADMIN;
  open(MAIL, "| $SEND_MAIL_CMD");
  print MAIL "To: $contact\n";
  print MAIL "From: $DEFAULT_SENDER\n";
  print MAIL "Subject: bbcheckdb.pl failure\n\n";
  print MAIL "bbcheckdb.pl failed...here is the error message:\n";
  print MAIL "$msg\n\n";
  print MAIL "                          -Nobody";
  close(MAIL);
  die "$msg";
};
__END__
