<?    
####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
############################################################################
# FUNCTIONS CALLED BY PHP SCRIPTS 
# Update: 8-22-02 Nick and Jerome - Declared language-substitution variables as
#   globals within each function as needed, so as to enable visibility upon
#   including a language substitution file defining these variables. 
# Update: 8-20-02 Nick and Jerome - Substituting out language-dependent text
#   strings with variables to allow multilingual support.
# 6-25-99 Gavin added standard comments to each of the functions 
# 6-22-99 adam continues the librarizing of the functions by isolating
#         those that print HTML
# 5-4-99 adam begins to librarize by abstracting the database
# 4-2-99 mike & yelena changed collegelinks to use new URL for Textbook lookup
#        and only show for current quarter
############################################################################

$FUNCTIONS_DEFINED = 1;
############################################################################
# Name: get_instru_info
# Purpose: To find information About the instructor.
# Usage: list($table_found_in, $profname, $profoffice, $profphone, $profemail, $profemail, "", $profinfo_hidden, $profinfo_updated, $profinfo_updated_uid, "") =# get_instru_info($fuid)
# Update: 2-5-00 mike added ClassWeb for TAs link to write_deptlinks subroutine
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function get_instru_info($fuid){
  # look up instructor's info in profsloc table first
  $profinfo = selectfirst ("select * from profsloc where uid='$fuid'");
  # if instructor's info is not in profsloc table, look it up in REGfaculty table
  if (!$profinfo) {
    $profinfo = selectfirst ("select * from REGfaculty where uid='$fuid'");

    # if instructor's info is not in REGfaculty table either, look it up in REGprof table which will only return instructor's name
    if (!$profinfo) { 
      $profinfo = selectfirst ("select * from REGprof where uid='$fuid'");
      
      # return instructor's info got from REGprof table
      return array("REGprof", $profinfo[reg_name], "", "", "", "", "", "No");
    } else {
        
      # return instructor's info got from REGfaculty table
      $loweremail = strtolower($profinfo[email]);
      return array("REGfaculty", $profinfo[fullname], $profinfo[Address], $profinfo[phone], $loweremail, $loweremail, "", "No");
    }
  } else {

    # check hide_email column in profsloc table
    $loweremail = strtolower($profinfo[email]);
    
## Mike replaced this with the section below so that only if hide_email="Yes"
## will it be hidden. This will cover us against blanks in the database.
#   if ($profinfo[hide_email]=="No") { 

#     # return instructor's info got from profsloc table with email   
#     return array("profsloc", $profinfo[fullname], $profinfo[office], $profinfo[phone], $loweremail, $loweremail, "", $profinfo[hide_email], $profinfo[updated], $profinfo[updateruid], "");
#   
#   } else { 
#   
#     # return instructor's info got from profsloc table without email 
#     return array("profsloc", $profinfo[fullname], $profinfo[office], $profinfo[phone], $loweremail, "N/A", "", $profinfo[hide_email], $profinfo[updated], $profinfo[updateruid], "");
#   }
    if ($profinfo[hide_email]=="Yes") { 
      # return instructor's info got from profsloc table without email 
## Mike changed this to display blank instead of N/A when no email.
#     return array("profsloc", $profinfo[fullname], $profinfo[office], $profinfo[phone], $loweremail, "N/A", "", $profinfo[hide_email], $profinfo[updated], $profinfo[updateruid], "");
      return array("profsloc", $profinfo[fullname], $profinfo[office], $profinfo[phone], $loweremail, "", "", $profinfo[hide_email], $profinfo[updated], $profinfo[updateruid], "");
    
    } else { 
    
      # return instructor's info got from profsloc table with email   
      return array("profsloc", $profinfo[fullname], $profinfo[office], $profinfo[phone], $loweremail, $loweremail, "", $profinfo[hide_email], $profinfo[updated], $profinfo[updateruid], "");
    }
  }
}
############################################################################

############################################################################
# Name: depttodirname
# Purpose: returns the dirname of a given dept
# Usage: $dirname = depttodirname($dept)
# Update: 9-28-1999 Adam Kaplan - writing
# Status: testing
function depttodirname ($dept) {
  $REGdept = selectfirst("select dirname from REGdept where dept = '$dept'");
  $dirname = $REGdept[dirname];
  return ($dirname);
}
############################################################################



############################################################################
# Name: get_ta_info
# Purpose: To find information about the TA.
# Usage: list($table_found_in, $taname, $taoffice, $taphone, $tamail, $taemail, "", $tainfo_hidden, $tainfo_updated, $tainfo_updated_uid, "") = get_ta_info($fuid)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function get_ta_info($fuid){
  
  # look up TA's info in profsloc table first
  $tainfo = selectfirst ("select * from profsloc where uid='$fuid'");

  # if TA's info is not in profsloc table, look it up in REGta table
  if (!$tainfo) { 
    $tainfo = selectfirst ("select * from REGta where uid='$fuid'");
     if (!$tainfo) {
       return array("-1");
     } 
    # return TA's info got from REGta table
    $loweremail = strtolower($tainfo[email]);
    return array("REGta", $tainfo[fullname], "", "", $loweremail, $loweremail, "", "No");
    
  } else {
        
    # check hide_email column in profsloc table
    $loweremail = strtolower($tainfo[email]);

    if ($tainfo[hide_email]=="No") {   
  
      # return TA's info got from profsloc table
      #return array("profsloc", $tainfo[fullname], $tainfo[office], $tainfo[phone], $loweremail, $loweremail, $tainfo[office_hours], $tainfo[hide_email], $tainfo[updated], $tainfo[updateruid]);
      return array("profsloc", $tainfo[fullname], $tainfo[office], $tainfo[phone], $loweremail, $loweremail, "", $tainfo[hide_email], $tainfo[updated], $tainfo[updateruid]);
    
    } else { 
    
      # return TA's info got from profsloc table
      #return array("profsloc", $tainfo[fullname], $tainfo[office], $tainfo[phone], $loweremail, "N/A", $tainfo[office_hours], $tainfo[hide_email], $tainfo[updated], $tainfo[updateruid], "");
      return array("profsloc", $tainfo[fullname], $tainfo[office], $tainfo[phone], $loweremail, "N/A", "", $tainfo[hide_email], $tainfo[updated], $tainfo[updateruid], "");
    } 
  }
}
############################################################################

############################################################################
# Name: get_sche_info
# Purpose: To find information about the scheduler of each department.
# Usage: list($table_found_in, $scheinfo_fullname, $scheinfo_phone, $email) = get_sche_info($uid)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function get_sche_info($uid){
  
  # look up scheduler's info in profsloc table first
  $scheinfo = selectfirst ("select * from profsloc where uid='$uid'");

  # if scheduler's info is not in profsloc table, look it up in REGfaculty table
  if (!$scheinfo) {
    $scheinfo = selectfirst ("select * from REGfaculty where uid='$uid'");

    # return scheduler's info got from REGfaculty table
    $loweremail = strtolower($scheinfo[email]);
    return array("REGfaculty", $scheinfo[fullname], $scheinfo[phone], $loweremail);
   
  } else {

    # return scheduler's info got from profsloc table
    $loweremail = strtolower($scheinfo[email]);
    return array("profsloc", $scheinfo[fullname], $scheinfo[phone], $loweremail);
  }
}
############################################################################

############################################################################
# Name: write_collegelinks 
# Purpose: To generate College links. 
# Usage: write_collegelinks($colspan, $bxref, $term, $srs, $summer, $dept, $coursenum, $subarea, $idxcrs) 
# Update: 8-15-99 Mike added $transparent, which drops bgcolor if it exists
# Update: 6/29/1999 Yelena addes $session to Textbooks URL if Summer Session
#         6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function write_collegelinks($colspan, $bxref, $term, $srs, $summer, $dept, $coursenum, $subarea, $idxcrs) { 
  global $transparent;
  global $_COLLEGELINKS;
  global $_REG_LISTING;
  global $_REG_COURSEINFO;
  global $_ASUCLATEXTBOOKS;
  global $_LIBRARYRESERVES;
 
  list($session, $reglisturl, $regdetailurl, $textbookurl, $reserveurl, $summer, $current) = collegelinks($term, $srs, $summer, $subarea, $idxcrs); 
  if ($transparent) {
    $bgcolor="";
  } else {
    $bgcolor = "bgcolor='#FFCC00'";
  }
  if ($colspan != 0) {
    echo "<tr><td align='center' colspan='$colspan' $bgcolor><font size='3' face='Arial,Helvetica'>$_COLLEGELINKS:</font></td></tr>";
  } #endif
  echo "<tr>";
  # decide the width of each column to be displayed in the table
  if ($bxref == 0) {
    $width = "33%";
  } else {
    $width = "27%";

    # display course title
    echo "<td align='left' width='20%' $bgcolor><font size='3' face='Arial,Helvetica'><B>";
    echo "$dept $coursenum";
    echo "</B></font></td>";
  }

  
  # display registrar listing
   echo "<td align='center' width='$width' $bgcolor><font size='3' face='Arial,Helvetica'>";
#  echo "<a href=$reglisturl >Registrar's Listing</a>";
   echo "<a href=$reglisturl >$_REG_LISTING</a>";
   echo "</font></td>";
               
   # display registrar's detail
   echo "<td align='center' width='$width' $bgcolor><font size='3' face='Arial,Helvetica'>";
#  echo "<a href=$regdetailurl >Registrar's Detail</a>";
   echo "<a href=$regdetailurl >$_REG_COURSEINFO</a>";
   echo "</font></td>";


   if ($textbookurl) { # display textbook link
     echo "<td align='center' width='$width' $bgcolor><font size='3' face='Arial,Helvetica'>";
     echo "<a href=$textbookurl > $_ASUCLATEXTBOOKS</a>";
     echo "</font></td>";
   } else {  # not current quarter, don't display textbook link
     echo "<td align='center' width='$width' $bgcolor><font size='3' face='Arial,Helvetica'>";
     echo "&nbsp;</font></td>";
   }

   if ($reserveurl) {
     echo "<td align='center' width='$width' $bgcolor><font size='3' face='Arial,Helvetica'>";
     echo "<a href=$reserveurl >$_LIBRARYRESERVES</a>";
     echo "</font></td>";
   } else {
     echo "<td $bgcolor>&nbsp;</td>";
   }
   echo "</tr>"; 
}
############################################################################

############################################################################
# Name: collegelinks
# Purpose: To fetch ucla links for write_collegelinks function.
# Usage: list($session, $reglisturl, $regdetailurl, $textbookurl, $reserveurl, $summer, $current) = collegelinks($term, $srs, $summer, $subarea, $idxcrs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function collegelinks($term, $srs, $summer, $subarea, $idxcrs){

  $reserve = selectfirst("select * from reserves where srs='$srs'");

  # calculate variables for summer sessions

  if ($summer) {  //get sessionsel if it's Summer Session
    $sumsess = selectfirst ("select session from REGsummer where term='$term' && srs='$srs'");
        
    if (!$sumsess) {  //no session value found, treat as non-summer
      $sumsess[] = "";
      $summer = "";  //without session, have to use regular Registrar URLs
    } else {  
      $x = $sumsess[session];
      $textbookurl = "http://proweb.asucla.ucla.edu/textbookstore/main.asp?remote=1&ref=ucla&term=$term$x&course=$srs&getbooks=Display+books";
      if (!($x && strchr("ABC", $x))) {  //A,B,C are only valid Summer Sessions
        $summer = "";   //without session, have to use regular URLs
      } #endif 
    } #endif 
  } else { # if not summer
    $textbookurl = "http://proweb.asucla.ucla.edu/textbookstore/main.asp?remote=1&ref=ucla&term=$term&course=$srs&getbooks=Display+books";
  } #endif
  $current_term = selectfirst("select term from current"); 
  $current=1; # starts off as true, this is current quarter, now check it
  if ($GLOBALS[term]!=$current_term[term]) { 
    $current=0; 
    $reserve[url]=''; 
    $textbookurl ='';
  }
  $reserveurl = $reserve[url];
  $session = $x;
  $reglisturl = "http://www.registrar.ucla.edu/schedule/$summer"; 
  $reglisturl .= "detselect.asp?termsel=$term&subareasel=$subarea$sessionsel&idxcrs=$idxcrs";
  $regdetailurl = "http://www.registrar.ucla.edu/schedule/$summer"; 
  $regdetailurl .= "subdet.asp?srs=$srs&term=$term";
  return array($session, $reglisturl, $regdetailurl, $textbookurl, $reserveurl, $summer, $current); 

} #collegelinks
############################################################################

############################################################################
# Name: combinedtitle 
# Purpose: To get combined course title and section title.
# Usage: $newtitle = combinedtitle($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function combinedtitle($term, $srs){
  
  # query classloc table to see if any local changes made to course title
  #   append an "*" to indicate the data from the local table 
  $title = selectfirst ("select title from classloc where term='$term' && srs='$srs'");
  
  # get course title and section title from input registrar data and concatenate the two
  if ($title[title] == "") {
    $title = selectfirst ("select coursetitle, sectitle from REGclasses where term='$term' && srs='$srs'");

    # get rid of coursenum from section title 
    $newsectitle = substr(strrchr($title[sectitle], ":"), 0);

    $newtitle = $title[coursetitle] . $newsectitle;
  } else {
    $newtitle = $title[title] . " *";
  }

  return $newtitle;

}
############################################################################

############################################################################
# Name: strwrap
# Purpose: To wrap a string at a given width.
# Usage: $newstring = strwrap($string, $width)
# Update: 10/11/1999 Adam Kaplan modified function to deal with HTML
#         6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production

function strwrap($string, $width) {
  # get the length of input string
  $n = strlen($string);
  
  # check if the string is shorter than the given width, if yes, just return
  #   the string but get rid of the white spaces at the beginning and the end
  if ($n <= $width) {
    $newstring = trim($string);
  } else {
    
    # initiate variables
    $pos = 0;
    $newstring = "";

    # recursively wrap the string
    while ($pos < $n) {

      # get the portion of string that no longer than the give width
      $string1 = substr($string, $pos, $width);  

      # get the last word of the part of the string and the length of the word
      $string2 = strrchr($string1, " ");
      $len = strlen($string2);

      # get the portion of string that not longer than the give width but
      #   without any word cut in the middle and add a line break at 
      #   its end also get rid of the white spaces at the beginning and the
      #   end
      $width1 = $width - $len;
      if ($width1 == 0) {
        $string3 = trim(substr($string, $pos, $width)) . "\n";
          # increase the position for next loop
        $pos = $pos + $width;
      } else {  
        $string3 = trim(substr($string, $pos, $width1)) . " \n";
          # increase the position for next loop
        $pos = $pos + $width1;
      } #endif
      
   
      # cancardenate the parts of string
      $string4 = $string4 . $string3;
     
    }

  # get rid of the line break (\n) at the last line
  $newstring = substr($string4, 0, strlen($string4)-1);
  }

  return $newstring;
}
############################################################################

############################################################################
# Name: timestamptodate
# Purpose: To break a database timestamp into a string in the format as the
#   output of PHP function date("M d Y H:i:s").
# Usage: $date = timestamptodate($timestamp)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# 

function timestamptodate($timestamp) {

  # year, numeric, 4 digits
  $Y = substr($timestamp, 0, 4);

  # month, numeric
  $m = substr($timestamp, 4, 2);

  # month, textual, 3 letters; i.e. "Jan"
  switch ($m) {
    case "01":
      $M = "Jan";
      break;
    case "02":
      $M = "Feb";
      break;
    case "03":
      $M = "Mar";
      break;
    case "04":
      $M = "Apr";
      break;
    case "05":
      $M = "May";
      break;
    case "06":
      $M = "Jun";
      break;
    case "07":
      $M = "Jul";
      break;
    case "08":
      $M = "Aug";
      break;
    case "09":
      $M = "Sep";
      break;
    case "10":
      $M = "Oct";
      break;
    case "11":
      $M = "Nov";
      break;
    case "12":
      $M = "Dec";
      break;
  }

  # day of the month, numeric
  $d = substr($timestamp, 6, 2);

  # hour, numeric, 24 hour format 
  $H = substr($timestamp, 8, 2);

  # minutes, numeric
  $i = substr($timestamp, 10, 2);

  # seconds, numeric
  $s = substr($timestamp, 12, 2);

  $date = "$M $d $Y $H:$i:$s";

  return $date;

}
############################################################################

############################################################################
# Name: findlinks
# Purpose: To find what links are present in course menu.
#          0 means no link for this page, 1 means link present.
# Usage: list($announce, $linkpage, $wwwboard) = findlinks($dir, $announce, $linkpage, $wwwboard) 
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
#          8/7/98 Matt Breeze
# Status: Production
# 

function findlinks($dir, $announce, $linkpage, $wwwboard) {

  $announce = 0;  # 0 means no link for this page, 1 means link present
  $linkpage = 0;
  $wwwboard = 0;
 
  $file = file("$dir/coursemenu.inc");  # should store coursemenu.inc in $file array

  while($line = each($file)) {
    if (strstr(strtolower($line[value]),'href="announce.')) { $announce = 1; }
    if (strstr(strtolower($line[value]),'href="linkpage.')) { $linkpage = 1; }
    if (strstr(strtolower($line[value]),'wwwboard')) { $wwwboard = 1; }
  }
# print "FUNCTION TEST \$dir=$dir, \$announce=$announce, \$linkpage=$linkpage, \$wwwboard=$wwwboard<br>\n";
  return array($announce, $linkpage, $wwwboard);
}
############################################################################

############################################################################
# Name: findnums
# Purpose: To get Dept and Coursenumber, including any cross-listeds
# Usage: $classnums = findnums($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
#         8/17/98 Matt Breeze - to be used by admin pages
# Status: Production

function findnums($term, $srs) {
  # get class info
  $class = selectfirst ("select * from REGclasses where term='$term' && srs='$srs'");
  if (!$class) {
    echo "<br>Input TERM or SRS number is not valid.</b>";
  } else {  # class data available for this class
    $qxref = query ("select * from REGxref where term='$term' && srs='$srs'");
    $numxref = numrows($qxref); # number of REGxref records

    # if no xref course, display detailed information and course title, 
    #   otherwise, display all the depts & coursenums plus course title
    if ($numxref == 0){ 
      $classnums = "$class[dept] $class[coursenum]";
    } else {
      $xreflist = "$class[dept] $class[coursenum]";
      while ($xref = walk ($qxref)) {
        $xreflist = "$xreflist / $xref[xdept] $xref[xcoursenum]";
      }
      $classnums = "$xreflist";
    }
  }
  return $classnums;
}
############################################################################

############################################################################
# Name: submenu
# Usage: submenu($type, $termsrs)
# Update: 9/30/1999 Yelena adding a homework type
#         6/25/1999 Gavin Yeung added standard comments to this function.
#         1/12/99 jason Created
# Status: Production
# Purpose: $type will be "announce","linkpage", or "wwwboard".

function submenu($type, $termsrs) {
  $BASE="/home/classweb";      # get program directory from Apache Environment  
  require "$BASE/customize.inc"; # load in customization info

   $announce = $wwwboard = $linkpage = 1;

   if($type == "wwwboard"){
      $wwwboard = 0;
      $urlprefix = '../';
      include("$BASE/ta/models/submenu.inc");
   } else if($type == "homework"){
      $wwwboard = 0;
      $urlprefix = '../';
      include("$BASE/ta/models/submenu.homework.inc");
   } else {
      if($type == "announce") $announce = 0;
      else $linkpage = 0;
      $urlprefix = '';
      include("$BASE/ta/models/submenu.inc");
   } 
}
############################################################################


############################################################################
# Name: expandterm
# Usage: list($quarter, $year) = expandterm($term) 
# Update: 5/26/99 Mike Franks - created
# Status: production
# Purpose: to split and expand $term into Fall
function expandterm($term) {
  global $_FALLQUARTER_UC;
  global $_WINTERQUARTER_UC;
  global $_SPRINGQUARTER_UC;
  global $_SUMMERQUARTER_UC;
  
  $qtr['F'] = "$_FALLQUARTER_UC";
  $qtr['W'] = "$_WINTERQUARTER_UC";
  $qtr['S'] = "$_SPRINGQUARTER_UC";
  $qtr['1'] = "$_SUMMERQUARTER_UC";
  $yr = substr($term, 0, 2);  # grab first two chars into $yr
  if ($yr > 90) {
    $yr += 1900;
  } else {
    $yr += 2000;
  }
  $xqtr = substr($term, -1);  # grab last char into $xqtr
  return array($qtr[$xqtr], $yr);
} #expandterm

############################################################################

############################################################################
# Name: gettermname
# Usage: list($quarter, $year) = expandterm($term) 
# Update: 01/18/01 Glen Stewart - created
# Status: production
# Purpose: to split and expand $term into Fall
function gettermname($term) {
  global $_FALL;
  global $_WINTER;
  global $_SPRING;
  global $_SUMMER;

  $qtr['F'] = "$_FALL";
  $qtr['W'] = "$_WINTER";
  $qtr['S'] = "$_SPRING";
  $qtr['1'] = "$_SUMMER";
  $xqtr = substr($term, -1);  # grab last char into $xqtr
  return array($qtr[$xqtr]);
} #gettermname

############################################################################


############################################################################
# Name: date_check
# Usage: date_check($file, $time) 
# Update: 09-17-02 Jerome - enabled date to be multilingual
# Update: 7/13/99 mike and yelena put $time back in, so TA sites will work
# Update: 7/6/99 Yelena added a 10 second range between the times
# Update: 5/26/99 Mike Franks - created
# Status: production
# Purpose: show date modified if it has been
function date_check($file,$time) {
    global $_JANUARY, $_FEBRUARY, $_MARCH, $_APRIL, $_MAY, $_JUNE, $_JULY, $_AUGUST, $_SEPTEMBER, $_OCTOBER, $_NOVEMBER, $_DECEMBER;

    # echo filemtime("$file"),"-$time<br>";  # for debugging
    if (abs(filemtime("$file") - "$time") > 5){
      # echo filemtime("$file"),"-$time<br>";  # for debugging
      # echo date("M d", filemtime("$file"));
      $monthofyear =  date("n", filemtime("$file"));
      if ($monthofyear == 1) {
        $monthofyear = $_JANUARY;
      } else if ($monthofyear == 2) {
        $monthofyear = $_FEBRUARY;
      } else if ($monthofyear == 3) {
        $monthofyear = $_MARCH;
      } else if ($monthofyear == 4) {
        $monthofyear = $_APRIL;
      } else if ($monthofyear == 5) {
        $monthofyear = $_MAY;
      } else if ($monthofyear == 6) {
        $monthofyear = $_JUNE;
      } else if ($monthofyear == 7) {
        $monthofyear = $_JULY;
      } else if ($monthofyear == 8) {
        $monthofyear = $_AUGUST;
      } else if ($monthofyear == 9) {
        $monthofyear = $_SEPTEMBER;
      } else if ($monthofyear == 10) {
        $monthofyear = $_OCTOBER;
      } else if ($monthofyear == 11) {
        $monthofyear = $_NOVEMBER;
      } else if ($monthofyear == 12) {
        $monthofyear = $_DECEMBER;
      }
      echo $monthofyear . date(" d", filemtime("$file"));
    }
}
############################################################################

############################################################################
# Name: write_quarter
# Usage: write_quarter ($term, $srs);
# Update: 7/1/99 Adam Kaplan - created
# Status: production
# Purpose: display current quarter 

function write_quarter ($term, $srs) {
  global $_SESSION;

  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info  
  list($qtrname,$fullyear) = expandterm($term);
  echo "<font color=\"$SCHOOL_COLOR2\" size='3' face='Arial,Helvetica'>";
  echo "$qtrname $fullyear"; # eg. SUMMER QUARTER 1999
  if (ereg("1$", $term, $res)) {
    $summer = "summer/"; 
  } #endif
  if ($summer) {
    $session = selectfirst ("select session from REGsummer where 
      term='$term' && srs='$srs'");
    $session = $session[session];
    echo " $_SESSION $session";
  } #endif
  echo "</font><br>\n";
} #write_quarter

############################################################################


############################################################################
########the following functions were written to abstract the SQL############
#######################functionality of the database########################
######### - adam 5/4/99 ####################################################
############################################################################

############################################################################
# Name: query
# Usage: $query_result = query($input)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Purpose: performs a mySQL lookup for user (based on term and srs if specified)
#          returns the query results. 
# Status: Production

function query ($input)
{
  return (mysql_query($input));  
}
############################################################################

############################################################################
# Name: walk
# Usage: $next_result = walk($query)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: gets next row from query (or first if no row has been taken).

function walk($query)
{
  return(mysql_fetch_array($query) );
}
############################################################################

############################################################################
# Name: numrows
# Usage: $number = numrows($query)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: gets number of rows in query.

function numrows($query)
{
  return(mysql_num_rows($query));
}
############################################################################

############################################################################
# Name: selectfirst 
# Usage: $result = selectfirst($list)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: gets first row only from a query. 

function selectfirst ($list)
{
  $q = query($list);   
  $res = walk($q);
  return($res);
}
############################################################################

############################################################################
# Name: qreset
# Usage: $result = qreset($query)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: resets the query.

function qreset($query)
{
  return (mysql_data_seek($query, 0));
} 
############################################################################


############################################################################
################ Adam Kaplan inserts these functions 5/4/99 ################
###################### to perform index.php operations #####################
############################################################################


############################################################################
# Name: write_classinfo 
# Usage: $class = write_classinfo($term, $srs, $page)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: get class info and print to screen.
#          class contains: $class[dept] $class[coursenum] $class[activity]
#          $class[section]: $class[newcoursetitle].

function write_classinfo ($term, $srs, $page) {
  global $_ERROR;

  $class = classinfo($term, $srs);
  if (!$class) {
    echo "$_ERROR";
  } else {
    if ($page != "Main") {
      echo "$page - $class[dept] $class[coursenum]";
    } else {
      echo "$class[dept] $class[coursenum] $class[activity] $class[section]: $class[newcoursetitle]";
    } #endif
  } #endif
  return $class;
} # write_classinfo
############################################################################

############################################################################
# Name: classinfo 
# Usage: $class = classinfo($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetch class info for write_classinfo1.

function classinfo ($term, $srs)
{
  $class = selectfirst ("select * from REGclasses where term='$term' && srs='$srs'");
  $newcoursetitle = combinedtitle($term, $srs);
  $class[newcoursetitle] = $newcoursetitle;
  return $class;
} #classinfo
############################################################################

############################################################################
# Name: callmethod
# Usage: $termsrs = callmethod($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: Determines if called from class directory or classweb/models.

function callmethod($term, $srs){
  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info

  $tempsubstr=ereg_replace("^/","",$BASE);
  if (strstr($SCRIPT_FILENAME, "$tempsubstr/prof/models"))
  {
    $nondir=1;
    $termsrs="?term=$term&srs=$srs";
  }
  else {
    $nondir=0;
    $termsrs="";
  }
  return $termsrs;
} # end of callmethod
############################################################################


############################################################################
# Name: write_allsrsinfo
# Usage: list($allsrs,$numxref,$qxref) = write_allsrsinfo($term, $srs, $class, $title)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetches and prints all srs info (has no helper function).

function write_allsrsinfo ($term, $srs, $class, $title) {
  global $_INVALID_DEPTSRS;


  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info
    if (!$class) {
      echo "<font color=\"$SCHOOL_COLOR2\" size='5' face='Arial,Helvetica'><b><br>$_INVALID_DEPTSRS<font></b>";
    } else {
      write_quarter($term, $srs); 
      echo "<font color=\"$SCHOOL_COLOR2\" size='5' face='Arial,Helvetica'><b>";
 
  $qxref = query("select * from REGxref where term = '$term' && srs = '$srs'");
  $numxref = numrows($qxref); // number of REGxref records

      # if no xref course, display detailed information and course title, 
      #   otherwise, display all the depts & coursenums plus course title
      if ($numxref == 0){ 
        echo "$class[dept] $class[coursenum]";
        if ($title == "Main") {
          echo " $class[activity] $class[section]";
          echo ": $class[newcoursetitle]";
        } else {
          echo "<br> $title";
        } #endif
      } else {
        if ($title == "Main") {
          $xreflist = "$class[dept] $class[coursenum] $class[activity] $class[section]";
        } else {
          $xreflist = "$class[dept] $class[coursenum]";
        } #endif   
        while ($xref = walk($qxref)) {
          if ($title == "Main") {
            $xreflist = "$xreflist / $xref[xdept] $xref[xcoursenum] $xref[xactivity] $xref[xsection]";
          } else {
            $xreflist = "$xreflist / $xref[xdept] $xref[xcoursenum]";
          } #endif
          $allsrs = "$allsrs,'$xref[xsrs]'";  # for joined class profs
        }
        
        echo "$xreflist <br>";
        if ($title == "Main") {
          echo "$class[newcoursetitle]";
        } else {
          echo "$title";
        } #endif
      } #endif
    } #endif

  return array($allsrs, $numxref, $qxref);

}  # write_allsrsinfo
############################################################################

############################################################################
# Name: write_meeting 
# Usage: write_meeting($term, $srs, $class)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetches and prints meeting time/place.

function write_meeting ($term, $srs, $class)
{
     global $_TBA;

     $qxref = query ("select * from REGxref where term='$term' && srs='$srs' && joinedclass is not null");
     $numxref = numrows($qxref); // number of REGxref records
     $qmeeting = query("select days, start, stop, bldg, room, startday, endday from REGmeetings where term= '$term' && srs = '$srs'");

     
     if($numxref != 0){
        echo "$class[dept] $class[coursenum] $class[activity] $class[section]: ";
     }

     if (!$qmeeting) {
        echo "$_TBA";
      } else {
        while ($meeting = walk($qmeeting)) {
          echo "$meeting[days]&nbsp;&nbsp;&nbsp; $meeting[start]-$meeting[stop]"; 
          echo "&nbsp;&nbsp;&nbsp; $meeting[bldg] $meeting[room]";
	  if ($meeting[endday]) {
	    echo "&nbsp;&nbsp;&nbsp; ($meeting[startday]-$meeting[endday])<br>";
	  } else {
	    echo "<BR>";
	  }
        }
     }

     if ($numxref != 0){ 
        while ($xref = walk($qxref)) {
          $qmeeting = query("select days, start, stop, bldg, room, startday, endday from REGmeetings where term = '$term' && srs='$xref[xsrs]'");

          $cstr = "$xref[xdept] $xref[xcoursenum] $xref[xactivity] $xref[xsection]";

          if (!$qmeeting) {
            echo "$_TBA";
          } else {
            while ($meeting = walk($qmeeting)) {
              echo "$cstr: $meeting[days]&nbsp;&nbsp;&nbsp; $meeting[start]-$meeting[stop]"; 
              echo "&nbsp;&nbsp;&nbsp; $meeting[bldg] $meeting[room]"; 
	      if ($meeting[endday]) {
	        echo "&nbsp;&nbsp;&nbsp; ($meeting[startday]-$meeting[endday])<br>";
	      } else {
	        echo "<BR>";
	      }
            }
          }
        }
     }

} #write_meeting
############################################################################

############################################################################
# Name: write_instructor
# Usage: list($numprofs, $qinstrinfo) = write_instructor($term, $srs, $allsrs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: Print instructor(s) information.

function write_instructor ($term, $srs, $allsrs)
{

      list($numprofs, $allsrs, $qinstrinfo) = instructor($term, $srs, $allsrs);
      
      $instrinfo = walk($qinstrinfo);
      
      if (!$instrinfo) {
        $instrinfo[]="";
      } else {
        
        $cntprof = 0;  

        while ($instrinfo) {

          $cntprof++; 
    
          # calling function to get instructor's info and the value returned is an array
          $profinfo = get_instru_info($instrinfo[uid]);

          if ($numprofs > 1) {
            $lname = substr($profinfo[1],0,strpos($profinfo[1],","));
            if ($lname == "") $lname = $profinfo[1];
            echo " $lname";
            if ($cntprof < $numprofs) { echo " /"; }
          } else {
            echo " $profinfo[1]";
          }

          $instrinfo = walk($qinstrinfo);
        }
      }
    return array($numprofs, $qinstrinfo);
} #write_instructor    
############################################################################

############################################################################
# Name: instructor
# Usage: list($numprofs,$allsrs,$qinstrinfo) = instructor($term, $srs, $allsrs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: Fetch instructor information for write_instructor.

function instructor($term, $srs, $allsrs) {
   $allsrs = "$srs" . $allsrs;
   $qinstrinfo = query("select distinct uid, reg_name from REGprof where term='$term' && srs = $srs && profcode < 3 order by reg_name");
   $numprofs = numrows($qinstrinfo); # number of instructors
   return array($numprofs, $allsrs, $qinstrinfo);

} #instructor
############################################################################

############################################################################
# Name: write_description
# Usage: list($newdesc, $newremarks) = write_description($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: print class description
  
function write_description($term, $srs)
{

  list($newdesc, $newremarks) = description($term, $srs);
  echo "<p align=left>$newdesc</p>";
  echo "<p align=left><big>$newremarks</big></p>";
  return array ($newdesc, $newremarks);
} #write_description
############################################################################

############################################################################
# Name: description
# Usage: description($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetches course description information for function write_description

function description ($term, $srs)
{
  # query classloc table to see if any local changes made to course description
  #   append an "*" to indicate the data from the local table 
  $locdesc = selectfirst ("select description, remarks from classloc where term = '$term' && srs = '$srs'");


  if ($locdesc[description]) {
    $newdesc = $locdesc[description] . " *";
  } else {
    #$desc = selectfirst ("select description from REGdesc where term = '$term' && srs = '$srs'");
    $desc = selectfirst ("select description from REGclasses where term = '$term' && srs = '$srs'");
    $newdesc = $desc[description];
  } #endif

  if ($locdesc[remarks]) {
    $newremarks = $locdesc[remarks];
  } else {
    $newremarks = "";
  } #endif
  
  return array($newdesc, $newremarks);

} # description
############################################################################

############################################################################
# Name: write_instructorinfo
# Usage: write_instructorinfo ($term, $srs, $class, $numprofs, $qinstrinfo, $fontarial3, $fontarial4)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: display instructor office, phone, email, office hours & homepg link

function write_instructorinfo ($term, $srs, $class, $numprofs, $qinstrinfo, $fontarial3, $fontarial4)
{
 
  global $HOMEPAGE_ICON;
  global $_INSTRUCTORS;
  global $_INSTRUCTOR;
  global $_OFFICE;
  global $_PHONENUMBER;
  global $_EMAILADDRESS;
  global $_OFFICEHOURS;
  global $_HOMEPAGE;
  
  if (!$class) {
    echo " ";
  } else {
    echo "<!-- table header --><table width='100%'><tr><td bgcolor=yellow>$fontarial3<b>";
    if ($numprofs > 1) {
    echo "$_INSTRUCTORS";
  } else {
    echo "$_INSTRUCTOR";
  }

    echo"</b></font></TD><td bgcolor=yellow><B>$fontarial3 $_OFFICE</B></font></TD>
                         <td bgcolor=yellow><B>$fontarial3 $_PHONENUMBER</B></font></TD>
                         <td bgcolor=yellow><B>$fontarial3 $_EMAILADDRESS</B></font></TD>
                         <td bgcolor=yellow><B>$fontarial3 $_OFFICEHOURS</B></font></TD></tr>";

    # protect against the case of no instructors listed
    if($numprofs != 0){
      # calling function to get instructor's info and the value returned is an array
      $resetqinstrinfo = qreset($qinstrinfo);
      $instrinfo = walk($qinstrinfo);
    }

    if (!$instrinfo) {
      $instrinfo[]="";
    } else {
      while ($instrinfo) {    //handles multiple instructors

        list($office_hours, $Profurl, $profinfo) = instructorinfo($term, $srs, $instrinfo[0]);
        if ($Profurl) { #if a URL exists for instructor
        echo "<td valign='top'>$fontarial4 <a href=\"$Profurl\">$profinfo[1]<img src=\"$HOMEPAGE_ICON\" align=absbottom border=0 alt=\"$_HOMEPAGE\"></a></font></td>\n";
        } else { #if no URL exists for instructor
          echo "<td valign='top'>$fontarial4 $profinfo[1] &nbsp;&nbsp;</font></td>\n";
        }
#       echo "<td valign='top'>$fontarial4 $profinfo[1]</font></td>\n";
        echo "<td valign='top'>$fontarial3 $profinfo[2]</font></td>\n";
        echo "<td valign='top'>$fontarial3 $profinfo[3]</font></td>\n";
        if (strstr($profinfo[5],"@")) {
          echo "<td valign='top'>$fontarial3 <a href='mailto:$profinfo[5]'>$profinfo[5]</font></a></td>\n"; 
        } else {
          echo "<td valign='top'>$fontarial3 $profinfo[5]</font></td>\n"; 
        }


          if (!$office_hours) { #if no office_hours
            echo "<td valign='top'>$fontarial3 </font></td>\n"; 
          } else { #if office hours          
            echo "<td valign='top'>$fontarial3 $office_hours</font></td>\n"; 
          }         
        echo "</tr>\n";
     
        $instrinfo = walk($qinstrinfo);
      } 
    }
  }

} # write_instructorinfo
############################################################################

############################################################################
# Name: instructorinfo
# Usage: list($office_hours, $Profurl, $profinfo) = instructorinfo ($term, $srs, $instructoruid)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetch information about an instructor for write_instructorinfo

function instructorinfo($term, $srs, $instructoruid){

   $profinfo = get_instru_info($instructoruid);
     ## Get Prof website if one exists
   $qProfsite = query ("select url from profurl where uid='$instructoruid'");
   $Profurl = "";
   if ($profsite = walk($qProfsite)) {
     $Profurl = "$profsite[url]";
   }
     # select the office hour for the individual course from hours 
   $query_new_hour = query ("select office_hours from hours where term='$term' && srs='$srs' && uid='$instructoruid'");
   $array_new_hour = walk ($query_new_hour);
   $office_hours = $array_new_hour[0];
   return array($office_hours, $Profurl, $profinfo);

} #instructorinfo
############################################################################

############################################################################
# Name: write_tainfo
# Usage: write_tainfo ($term, $srs, $allsrs, $fontarial3, $fontarial4)
# Update:  9/30/1999 Adam Kaplan modified query method to speed up loading
#          6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: print TA info

function write_tainfo ($term, $srs, $allsrs, $fontarial3, $fontarial4)
{
  global $_TEACHINGASSTS;
  global $_TEACHINGASST;
  global $_OFFICE;
  global $_PHONENUMBER;
  global $_EMAILADDRESS;
  global $_OFFICEHOURS;

  # get TA's information from REGta table

  $allsrs = "$srs" . $allsrs;
  $qtaquery = "select distinct REGta.uid, REGta.fullname, REGta.email, REGclasses.srs from REGta, REGclasses where ";
  $srses = split(",\s*" , $allsrs );
  $numsrs = sizeof($srses);
  if ($numsrs > 1 ) {
    $qtaquery .= "(";
    $notfirst = 0;
    for($i = 0; $i < $numsrs; $i++) {
      $currentsrs = $srses[$i];
      $currentsrs = ereg_replace ("'","",$currentsrs);
      $currentsrs = ereg_replace ("\"","",$currentsrs);
      if ($notfirst == 1) {
        $qtaquery .= " || ";
      } #endif
      $qtaquery .= "REGclasses.srs = \"$currentsrs\"";
      $notfirst = 1;
    } #endwhile
    $qtaquery .= ") && ";
  } else {
    $currentsrs = ereg_replace ("'","",$currentsrs);
    $currentsrs = ereg_replace ("\"","",$currentsrs);
    $currentsrs = $srses[0];
    $qtaquery .= "REGclasses.srs = '$currentsrs' && ";
  } #endif
        
        
  $qtaquery .= "REGta.term='$term' && REGclasses.term='$term' && REGta.dept = REGclasses.dept && REGta.coursenum=REGclasses.coursenum && REGta.relsecidx=REGclasses.secidx order by REGta.fullname";
        
  $qtainfo = query($qtaquery);

  $numtas = numrows($qtainfo); // number of TAs

  $tainfo = walk($qtainfo);
  
  if (!$tainfo) {
    $tainfo[] = "";
  } else {
    # continue to print table, list title of TA section of the table
    # create some space between entries of instructors and TAs -->
    echo "<tr><td></td></tr>\n";
    echo "<tr><td></td></tr>\n";
    echo "<tr><td></td></tr>\n";
    echo "<tr><td bgcolor=yellow>$fontarial3 <b>";
    
    if ($numtas > 1) {
      echo "$_TEACHINGASSTS";
    } else {
      echo "$_TEACHINGASST";
    }

    echo "</b></font></TD><td bgcolor=yellow><B>$fontarial3 $_OFFICE</B></font></TD>
                          <td bgcolor=yellow><B>$fontarial3 $_PHONENUMBER</B></font></TD>
                          <td bgcolor=yellow><B>$fontarial3 $_EMAILADDRESS</B></font></TD>
                          <td bgcolor=yellow><B>$fontarial3 $_OFFICEHOURS</B></font></TD></tr>";

    # print each TA's info
    while ($tainfo) {
      $uid = $tainfo[uid];
      if ($printed[$uid] != 1) { 
        $printed[$uid] = 1;
        list($office_hours, $TAurl, $sectionstoprint, $moretainfo) = tainfo ($term,$srs,$tainfo);
        if ($TAurl) {
          echo "<td valign='top'>$fontarial3 <a href=\"$TAurl\">$moretainfo[1] &nbsp;&nbsp;($sectionstoprint)</a></font></td>\n";
        } else {
          echo "<td valign='top'>$fontarial3 $moretainfo[1] &nbsp;&nbsp;($sectionstoprint)</font></td>\n";
        }
        echo "<td valign='top'>$fontarial3 $moretainfo[2]</font></td>\n";
        echo "<td valign='top'>$fontarial3 $moretainfo[3]</font></td>\n";
        if (strstr($moretainfo[5],"@")) {
          echo "<td valign='top'>$fontarial3 <a href='mailto:$moretainfo[5]'>$moretainfo[5]</font></a></td>\n"; 
        } else {
          echo "<td valign='top'>$fontarial3 $moretainfo[5]</font></td>\n"; 
        }
        #echo "<td valign='top'>$fontarial3 $moretainfo[6]</font></td>\n"; 
  
  
        if (!$office_hours) {
          echo "<td valign='top'>$fontarial3 </font></td>\n"; 
        } else {           
          echo "<td valign='top'>$fontarial3 $office_hours</font></td>\n"; 
        }         
        echo "</tr>\n";
      }
      $tainfo = walk($qtainfo);
    }
  }

} # write_tainfo
############################################################################

############################################################################
# Name: tainfo
# Usage: list($array_new_hour[office_hours], $TAurl, $sectionstoprint, $moretainfo) = tainfo($term, $srs, $tainfo)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetch information about a TA for write_tainfo

function tainfo ($term, $srs, $tainfo) {

  $moretainfo = get_ta_info($tainfo[uid]);

    ## Get list of sections this TA is teaching
  $qtasections = query ("select distinct section from REGta
    where term='$term' && uid='$tainfo[uid]'");
     # Adam puts a "distinct" into this query 10-25-99
     # to stop the repeated sections problem
  $sectionstoprint = "";
  while ($tasections = walk($qtasections)) {
    if ($sectionstoprint) {
      $sectionstoprint = "$sectionstoprint $tasections[section]";
    } else {
      $sectionstoprint = "$tasections[section]";
    } #endif
  } #endwhile
    
    ## Get TA website if one exists
  $qTAsite = query ("select url from sections
    where term='$term' && srs='$srs' && uid='$tainfo[uid]'");
  $TAurl = "";
  if ($tasite = walk($qTAsite)) {
    $TAurl = "http://$tasite[url]";
  } #endif
  
    # select the office hour for the individual course from hours     
  $query_new_hour = query ("select office_hours from hours where term='$term' && srs='$tainfo[srs]' && uid='$tainfo[uid]'");

  $array_new_hour = walk($query_new_hour);
          
  return array($array_new_hour[office_hours], $TAurl, $sectionstoprint, $moretainfo);

} #tainfo
############################################################################

############################################################################
# Name: write_finalexam 
# Usage: list($newexam, $exam) =  write_finalexam($term, $srs, $class)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: print final exam info

function write_finalexam ($term, $srs, $class)
{
  $qxref = query ("select * from REGxref where term='$term' && srs='$srs' && joinedclass is not null");
  $numxref = numrows($qxref); // number of REGxref records
  list($newexam, $exam) = finalexam($term, $srs);
  if($numxref != 0){ #if cross listed classes
    echo "<b>$class[dept] $class[coursenum] $class[activity] $class[section]:</b> ";
  }
  echo "($class[examcode]) $newexam<br>\n";
  if ($numxref != 0){ 
    while ($xref = walk($qxref)) { #for each crosslisted class
      list($newexam, $exam) = finalexam($term, $xref[xsrs]);
      $cstr = "<b>Final Exam Code: $xref[xdept] $xref[xcoursenum] $xref[xactivity] $xref[xsection]:</b>";
      echo "$cstr ($exam[examcode]) $newexam<br>\n";
    } #endwhile 
  } #endif
  echo "</font><p>\n";
  return array($newexam, $exam);
} #write_finalexam
############################################################################

############################################################################
# Name: finalexam 
# Usage: list($newexam, $exam) = finalexam($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: get exam information for a class for function write_finalexam

function finalexam($term, $srs){ 

  # query classloc table to see if any local changes made to examcode
  #   append an "*" to indicate the data from the local table 

  $locexam = selectfirst ("select exam from classloc where term = '$term' && srs = '$srs'");
  $exam = selectfirst ("select examcode, examcodetext from REGclasses where term = '$term' && srs = '$srs'");
  if ($locexam[exam] == "") {
    $newexam = $exam[examcodetext]; // get examcodetext from registrar data 
  } else {
    $newexam = $locexam[exam] . " *";   
  } 
     
  return array ($newexam, $exam); 

} #finalexam


############################################################################
# Name: linkuc 
# Usage: linkuc($term, $srs, $summer, $class, $session_code)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: uses write_collegelinks to print College links

function linkuc ($term, $srs, $summer, $class, $session_code)
{
  $qxref = query ("select * from REGxref where term = '$term' && srs = '$srs'");
  $numxref = numrows($qxref); // number of REGxref records
  # calculating variables to generate links
  $subarea = ereg_replace(" ","+",$class[dept]);
  $idxcrs = ereg_replace(" ","+",$class[crsidx]);  
  if ($summer) { 
    $idxcrs .= "++++++++";           # pad out with ++++  mike added 5/26/99
    $idxcrs = substr($idxcrs,0,8);   # pad out with ++++  mike added 5/26/99
    $idxcrs .= "+$session_code";
  } #endif

  if (!$class) {
    $class[] = "";  # Course SRS number is not valid.
  } else {  #check if there's any xref courses
     
    if ($numxref == 0) { # there is no xref 
      $bxref = 0;
      # calling function write_collegelinks to generate links and display results
      if ($COLLEGELINKS){
        write_collegelinks(4, $bxref, $term, $srs, $summer, $class[dept], $class[coursenum], $subarea, $idxcrs);
      }
    } else {  # there is xref
      $bxref = 1;
      # calling function write_collegelinks to generate links and display results
      if ($COLLEGELINKS){
        write_collegelinks(5, $bxref, $term, $srs, $summer, $class[dept], $class[coursenum] . "-". $class[section], $subarea, $idxcrs); 
      }
      # reset the pointer to the start of query result from REGxref table
      $resetqxref = qreset($qxref);
      # calculate links for xref courses
      while ($xref = walk($qxref)) {  
        # recalculating variables to generate links
        $subarea = ereg_replace(" ","+",$xref[xdept]);
        $idxcrs = ereg_replace(" ","+",$xref[xcrsidx]);  
        if ($summer) { 
          $idxcrs .= "++++++++";    
          $idxcrs = substr($idxcrs,0,8);  
          $idxcrs .= "+$session_code";
        } #endif
        # calling function write_collegelinks to generate links and display results
        if ($COLLEGELINKS){
          write_collegelinks(0, $bxref, $term, $xref[xsrs], $summer, $xref[xdept], $xref[xcoursenum] . "-" . $xref[xsection], $subarea, $idxcrs); 
	}
      }
    }
  }
}  # linkuc
############################################################################

############################################################################
# Name: write_linkdept 
# Usage: write_linkdept($term, $class)
# Update: 10/25/99 Mike changed link to ClassWeb Feedback
# Update: 8/12/99 Yelena added link to feedback_classweb.php called Comments
#         6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose:  print department links

function write_linkdept ($term, $class)
{
  global $_CLASSWEB_FEEDBACK;
  global $_CLASSWEBSITES;
  global $_ADMINISTRATION;
  global $_MAINPAGE_EDIT;
  global $_UPDATEDDAILY;

  # initialize global variables, load functions and connect to database
  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info

  echo "<a href=\"$CLASSES/feedback_classweb.php\"> $_CLASSWEB_FEEDBACK</a> /";
   # get dept info from REGdept table
  $newdepturl = linkdept($class[dept]);
  echo "<a href=\"$newdepturl\"> $class[dept]</a> /";

  # if there's xref courses, display those dept url as well
  $qxref = query ("select * from REGxref where term = '$term' && srs = '$srs'");
  $numxref = numrows($qxref); // number of REGxref records
  if ($numxref > 0) {

    # calculate links for xref courses
    while ($xref = walk($qxref)) {
      # get dept info from REGdept table
      $newdepturl = linkdept($xref[xdept]);
        # display dept link
      echo "<a href=\"$newdepturl\"> $xref[xdept]</a> /";
    } #endwhile

  } #endif

  # display lists of all courses in Social Science taught in current quarter
  echo "<a href=\"http://$SERVER_NAME$ENTRY/classlist.php?term=$term\"> $term $_CLASSWEBSITES</a> /"; 

  # Uncomment whatever links you need...
  #echo "<a href=\"$DEPT_URL\">$DEPT_NAME</a> /\n"; 
  #echo "<a href=\"$DIVISION_URL\">$DIVISION_NAME</a> /\n";
  echo "<a href=\"$SCHOOL_URL\">$SCHOOL_NAME</a><br>\n";

# commented out for now.  not implimented yet
# echo "<a href=\"/classes/ta/\">ClassWeb for TAs</a> /\n";

  echo "<a href=\"admin/\">$_ADMINISTRATION</a> /\n";
  echo "<a href=\"admin/?type=main\">$_MAINPAGE_EDIT</a> \n";
# echo "<a href=\"admin/stub.pl\">Old Admin</a>\n";
  echo "</font></p>\n";

  echo "<!-- closing -->\n";
  echo "<p align=\"center\"><font size=\"1\" face=\"Arial,Helvetica\">\n";
  echo "$_UPDATEDDAILY"; 

} #write_linkdept
############################################################################

############################################################################
# Name: linkdept
# Usage: $newdepturl = linkdept($dept)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: fetches department URL for a department

function linkdept ($dept){
  # initialize global variables, load functions and connect to database
  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info

  # get dept info from REGdept table
  $newREGdept = selectfirst ("select url from REGdept where dept='$dept'");
  if ($newREGdept) {
    if (substr($newREGdept[url], 0, 1) == "/") {
        # affix "http://www.server.name" to each url from REGdept table
      $newdepturl = "http://$SERVER_NAME$newREGdept[url]";
    } else {
      $newdepturl = $newREGdept[url];
    } #endif
  }#endif
  return $newdepturl;
} #linkdept


############################################################################
######################## Adam Kaplan May 5, 1999 ###########################
#################### adds functions for linkpage.php #######################
############################################################################

############################################################################
# Name: getcurrentlinkmode
# Usage: $mode = getcurrentlinkmode ($term,$srs,$site)
# Update: 9.1.1999 Adam Kaplan - created
# Status: Writing
# Purpose: find a class's current linkmode (1 = database, 0 = flatfile)
# 

function getcurrentlinkmode ($term,$srs,$site) { 
  $q = query("select * from linkold where term = '$term' && srs = '$srs'
   && site='$site'"); 
  if (numrows($q) == 0) {
    $mode = 1;
  } else {
    $mode = 0;
  }
  return $mode;
} #getcurrentlinkmode
############################################################################

############################################################################
# Name: linkfinish
# Usage: linkfinish ($class, $qxref, $numxref, $linksdir)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: finish links.

function linkfinish ($class, $qxref, $numxref, $linksdir)
{
   ###  include links for home dept
  $linksfile = strtolower(ereg_replace(" ","",$class[dept]));
  # remove blanks and change to lower case
  if (file_exists("$linksdir/$linksfile.htm")) {
    include("$linksdir/$linksfile.htm");
  }
  if ($numxref > 0) {  
   # handle all crosslisted classes here by getting
   # dirname and checking links directory for link file.
    $resetqxref = qreset($qxref);
    while ($xref = walk($qxref)) {
      $linksfile = strtolower(ereg_replace("( |&)","",$xref[xdept]));
      if (file_exists("$linksdir/$linksfile.htm")) {
        include("$linksdir/$linksfile.htm");
      }
    }
  }

} #linkfinish
############################################################################



############################################################################
# Name: write_recursivelinks
# Usage: recursiveprint($parent, $term, $srs, $site) where $parent = '' for full table
# Update: 7/30/99 Adam Kaplan - created
# Status: Production
# Purpose: print the link table recursively starting at a parent

function write_recursivelinks ($parent, $term, $srs, $site) {
  if ($parent == '') {
    $q = query("select * from links where term = '$term' && srs = '$srs' && site = '$site' && parent = '0' order by name");
  } else { # not first level
    $q = query("select * from links where term = '$term' && srs = '$srs' && site = '$site' && parent = '$parent' order by name");
  } #endif

  if (numrows($q) == 0) {
    return;
  } #endif
  print "<UL>\n";
  while ($current = walk($q)) {
    $selectpre = "";
    $selectpost = "";
    $number = $current[ID];
    print "<LI>\n";
    $pval = "";
    if ($current[parent] != 0) {
      $pval = $current[parent];
    } #endif
    $kids = query("select ID from links where term='$term' && srs='$srs' && site = '$site' && parent='$current[ID]'");
    if (($current[parent] == 0) || (numrows($kids) != 0)) {
      $selectpre = "<B>";
      $selectpost = "</B>";
    }
    if ($current[url]) {
      print "<A HREF=\"$current[url]\">$selectpre$current[name]$selectpost</A>";
    } else {
      print "<B>$selectpre$current[name]$selectpost</B>";
    } #endif
    if ($current[description]) {
      print " - $current[description]<BR>\n";
    } #endif
    write_recursivelinks($number, $term, $srs, $site);
  } #endwhile
  print "</UL>\n";
} #recursiveprint
############################################################################


############################################################################
################# Functions for TA site
################ functions corresponding to ta/index.php
############################################################################

############################################################################
# Name: ta_siteinfo
# Usage: list($sectionstoprint, $ta_lname, $moretainfo) = ta_siteinfo ($term, $ta_uid) 
# Update: 9/20/1999 Mike added ltrim on sectionstoprint to remove leading space
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: get TA information

function ta_siteinfo ($term, $ta_uid) {
  global $_TBA;


  $moretainfo = get_ta_info($ta_uid);

  $ta_lname = substr($moretainfo[1],0,strpos($moretainfo[1],","));
  if ($ta_lname == "") $ta_lname = $moretainfo[1];

     ### Change this section to handle all sections
  $qtasections = query ("select section from REGta where term='$term' && uid='$ta_uid'");
  $sectionstoprint = "";
  while ($tasections = walk($qtasections)) {
    $sectionstoprint = "$sectionstoprint $tasections[section]";
  }
  $sectionstoprint = ltrim($sectionstoprint); # remove leading space
  if ($sectionstoprint == "") { $sectionstoprint = "$_TBA"; }
  
  return array($sectionstoprint, $ta_lname, $moretainfo);

} # ta_siteinfo
############################################################################

############################################################################
# Name: tacallmethod
# Usage: $termsrs = tacallmethod($term, $srs)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: Determines if called from class directory or classweb/models.

function tacallmethod($term, $srs){
  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info
  $tempsubstr=ereg_replace("^/","",$BASE);    
  if (strstr($SCRIPT_FILENAME, "$tempsubstr/ta/models"))
  {
    $nondir=1;
    $termsrs="?term=$term&srs=$srs";
  }
  else {
    $nondir=0;
    $termsrs="";
  }
  return $termsrs;
} # end of tacallmethod
############################################################################

############################################################################
# Name: write_tacoursetitle
# Usage: $class = write_tacoursetitle($ta_lname, $term, $srs, $class, $sectionstoprint, $page)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: Prints course title and information to ta main page.

function write_tacoursetitle($ta_lname, $term, $srs, $class, $sectionstoprint, $page)  {
  global $_ERROR;

  if (!$class) {
    echo "$_ERROR";
  } else {
    # get term image size and display image
    $class[newcoursetitle] = combinedtitle($term, $srs);
    if ($page != "Main") {
      echo "$page - $class[dept] $class[coursenum] (TA=$ta_lname Sections: $sectionstoprint)";
    } else {
      echo "TA Site: $class[dept] $class[coursenum] (TA=$ta_lname Sections: $sectionstoprint)";
    } #endif
  } #endif

  return $class;
} # write_tacoursetitle
############################################################################

############################################################################
# Name: write_taheader
# Usage: write_taheader ($term, $srs, $class, $ta_lname, $moretainfo, $sectionstoprint, $title)
# Update: 6/25/1999 Gavin Yeung added standard comments to this function.
# Status: Production
# Purpose: prints header to ta main page

function write_taheader ($term, $srs, $class, $ta_lname, $moretainfo, $sectionstoprint, $title) {
  global $_ERROR;
  global $_TASITE;
  global $_TA;
  global $_SECTIONS;

  $BASE="/home/classweb";      # get program directory from Apache Environment
  require "$BASE/customize.inc"; # load in customization info
 if (!$class) {
    echo "$_ERROR";
  } else {
    # get term image size and display image
      write_quarter($term, $srs); 

    echo "<font color=\"$SCHOOL_COLOR2\" size='5' face='Arial,Helvetica'><b>\n";
    # get newcoursetitle as combination of coursetitle and sectitle
    if ($title == "Main"){ 
      $newcoursetitle = combinedtitle($term, $srs);
      echo "$_TASITE: $class[dept] $class[coursenum] $newcoursetitle";
      echo "<br>$_TA: $moretainfo[1]";
      echo "<br>$_SECTIONS: $sectionstoprint";
    } else { 
      echo "$class[dept] $class[coursenum] - TA: $ta_lname ($sectionstoprint)\n";
      echo "<br>$title\n";
    } #endif

    echo "</b></font>\n";
  } #endif

} # write_taheader
############################################################################

############################################################################
# Name: write_tatable
# Usage: write_tatable($fontarial3, $class);
# Update: 6.28.99 Adam Kaplan - created
# Status: production
# Purpose: writes the basic ta table to the screen
function write_tatable ($fontarial3, $class) {
  global $_TEACHINGASST;
  global $_OFFICE;
  global $_PHONENUMBER;
  global $_EMAILADDRESS;
  global $_OFFICEHOURS;

  if (!$class) {
    echo " ";
  } else {
    echo "<!-- table header --><table border=0 width='100%'><tr>";
    echo "<td><b>$fontarial3 $_TEACHINGASST</b></font></td>";
    echo"</b></font></TD><td><b>$fontarial3 $_OFFICE</b></font></td>
                         <td><b>$fontarial3 $_PHONENUMBER</b></font></td>
                         <td><b>$fontarial3 $_EMAILADDRESS</b></font></td>
                         <td><b>$fontarial3 $_OFFICEHOURS</b></font></td></tr>";
  } #endif


} # write_tatable
############################################################################

############################################################################
# Name: write_tatablefill
# Usage: write_tatablefill ($term, $srs, $ta_uid, $fontarial3, $moretainfo);
# Update: 6.28.99 Adam Kaplan - created
# Status: production
# Purpose: display TA's information into table

function write_tatablefill ($term, $srs, $ta_uid, $fontarial3, $moretainfo){
  echo "<td valign='top'>$fontarial3 $moretainfo[1]</font></td>\n";
  echo "<td valign='top'>$fontarial3 $moretainfo[2]</font></td>\n";
  echo "<td valign='top'>$fontarial3 $moretainfo[3]</font></td>\n";
  if ($moretainfo[5] && strstr($moretainfo[5],"@")) {
    echo "<td valign='top'>$fontarial3 <a href='mailto:$moretainfo[5]'>$moretainfo[5]</font></a></td>\n";
  } else {
    echo "<td valign='top'>$fontarial3 $moretainfo[5]</font></td>\n";
  } #endif

    # select the office hour for the individual course from hours
  $array_new_hour = selectfirst ("select office_hours from hours where term='$term' && srs='$srs' && uid='$ta_uid'");

  if (!$array_new_hour[office_hours]) {
    echo "<td valign='top'>$fontarial3 </font></td>\n";
  } else {
    echo "<td valign='top'>$fontarial3 $array_new_hour[office_hours]</font></td>\n";
  }
  echo "</tr>\n";

} # write_tatablefill
############################################################################

############################################################################
# Name: write_tachange
# Usage: write_tachange($newcoursetitle, $newdesc, $newexam);
# Update: 6.28.99 Adam Kaplan - created
# Status: production
# Purpose: prints a change message (uses possible future variables)

function write_tachange ($newcoursetitle, $newdesc, $newexam){
  global $_INFOCHANGEDFROMOFFICIAL;
  
  $star1 = strstr($newcoursetitle, "*");
  $star2 = strstr($newdesc, "*");
  $star3 = strstr($newexam, "*");
  #TODO: The registrar links were removed for TA sites. Should they be in?
  #If not, this message needs to be ammended.
  if ($star1 || $star2 || $star3) {
    echo "* $INFOCHANGEDFROMOFFICIAL\n";
  } #endif
} # write_tachange
############################################################################

?>
