#!/usr/local/bin/perl
#########################################
# Name: hits.pl
# Usage: run at command line or as a cronjob
# Update: 08-10-00 kenny - removed hard coded vars; adapted for opensource
# Status: production
# Purpose: goes thru web server logs and compiles count of main class page hits per class

# if hits.pl is run from a cronjob or the command line
# following code is necessary to determine the BASE prog. dir
unless ($BASE) { # If $BASE doesn't exist, use calling path or current directory
  use File::Basename;
  $BASE = dirname($0); # set $BASE to this program's calling path
  unless ($BASE) {
    # for programs called from this directory via telnet command with . on path
    $BASE=".";  # set to current directory if it still doesn't exist
  }
}

require "$BASE/customize.pl";  # important vars set here
require "$BASE/dbvars.pl";     # database vars set here
require("$BASE/dbopen.pl");

%MONTHS = (
"Jan" => 1,
"Feb" => 2,
"Mar" => 3,
"Apr" => 4,
"May" => 5,
"Jun" => 6,
"Jul" => 7,
"Aug" => 8,
"Sep" => 9,
"Oct" => 10,
"Nov" => 11,
"Dec" => 12
);

$base = "$DOCUMENT_ROOT$CLASSES";

$cmd = "select term from current";
$sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
$sth->execute || die "Can't execute query: $sth->errstr\n";
$cq = ($sth->fetchrow_array)[0];

$cmd = "select valid from current";
$sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
$sth->execute || die "Can't execute query: $sth->errstr\n";
$nq = ($sth->fetchrow_array)[0];
$nq =~ /$quarter (\w\w\w)/;
$nq = $1;

foreach $quarter($cq, $nq){
@urls = &get_url_list($quarter);

foreach $url(@urls) {
  if ($url =~ m#$SERVER_NAME(.*)#i) {
    $class = $1;    # $oclass will be used to get back 
    ($endchar) = ($class =~ m#(.)$#);
    if ($endchar ne "/") {
      if ($class =~ /index.htm/) {
        $class =~ s/index.html//i;
        $class =~ s/index.htm//i;
      } elsif ($class =~ /html?$/i) {  # ends with html or htm
        # leave ending alone 
      } else {
        $class .= "/";     # add slash to end for conformity
      } 
    }

    chop($class) if($class =~ /.*\//);
    $classes{$class} = 0;
  }
}

#######################################
# suck in hits.dat and get 1stline lastline as well as set previous class hits
$LOG=$ACCESS_LOG;
open (HITS,"<$base/$quarter/hits.dat");
@previous_hits = <HITS>;
close(HITS);

foreach $prev_hit (@previous_hits) {
  if ($prev_hit =~/FIRSTLINE:(.*)/) {
    $testline = $1;
    $firstline{$LOG} = "$testline\n";
  } elsif ($prev_hit =~/LASTLINE:(.*)/) {
    $testline = $1;
    $lastline{$LOG} = "$testline\n";
  } elsif ($prev_hit =~/^LOG PROCESSING/) {
    next;
  } elsif ($prev_hit =~/(.*) (\d+)/) {
    $dir = $1;
    $count = $2; 
    $classes{$dir} = $count;
  }
}

  system "cp $base/$quarter/hits.dat $base/$quarter/hits.dat.old" if(-e "$base/$quarter/hits.dat");
  open(OUT,">$base/$quarter/hits.dat") || die("Couldn't open $base/$quarter/hits.dat");
  $logtime = `date`;
  print OUT "LOG PROCESSING STARTED AT $logtime";

  print "Reading Log: $LOG\n" if $verbose;
  if ($LOG =~ /\.Z/)      { open(LOG,"uncompress -c $LOG |"); }
  elsif ($LOG =~ /\.gz/)  { open(LOG,"gzip -d -c $LOG |"); }
  elsif ($LOG =~ /\.zip/) { open(LOG,"unzip -p $LOG |"); }
  else { open (LOG,"$LOG"); }

  # Make sure we don't re-process lines!
  $_=<LOG>;
  # Yup, I've seen this log before
  if (($firstline{$LOG}) && ($_ eq $firstline{$LOG})) {
    print "   I've Seen this log file before.\n" if $verbose;
    print OUT "LOG PROCESSING I've Seen this log file before.\n";
    while (<LOG>) { 
      ($_ eq $lastline{$LOG}) && last; 
      }
    print "   Starting at line $.\n" if $verbose;
    print OUT "LOG PROCESSING Starting at line $.\n";
  # Nope, this is a new log
  } else { 
    print "   Looks like a new log file.\n" if $verbose;
    print OUT "LOG PROCESSING Looks like a new log file.\n";
    $firstline{$LOG}=$_; 
    seek(LOG,0,0);
  }

  # Okay, we're at the right spot...now read the new lines
LOGLOOP:
    while (<LOG>) {
    (($. % 10000)==0) && (print "$when\n") if $verbose;  # print date every 500 lines
    $lastline{$LOG}=$_;

    unless
    (($site,$user,$when,$request,$status)=
    /^(\S+)\s*\S+\s*(\S+)\s*\[([^\]]+)]\s*\"([^\"]+)"\s*(\S+)\s*\S+\s*\"\s*[^\"]*\s*\"\s*\"\s*[^\"]*\s*\"\s*$/o) {
      ###$lineerrors++;
      next;
    }

    ($page) = ($request =~ m|[^\/]*(\S*)|);

    ###unless
    ###(($site,$rfc931,$user,$when,$request,$status,$bytes,$from,$agent)=
    ###/^(\S+)\s*(\S+)\s*(\S+)\s*\[([^\]]+)]\s*\"([^\"]+)"\s*(\S+)\s*(\S+)\s*\"\s*([^\"]*)\s*\"\s*\"\s*([^\"]*)\s*\"\s*$/o)
    ###  { $lineerrors++; next; }
    ###($page) = ($request =~ m|[^\/]*(\S*)|);

    # If you've got other plans, let's do them!
    &process if (defined &process);

    # Lots of ways to decide if we want this line or not
    next if (($IGNORESITE) && ($site=~ /$IGNORESITE/io));
    next if (($IGNOREPAGE) && ($page=~ /$IGNOREPAGE/io));
    next if (($STRING)     && ($page!~ /$STRING/io));

    # Check for those pesty (null) entries that sometimes appear
    if ($site =~ /\(null\)/i) { next LOGLOOP; } ###$null++; next LOGLOOP; }

    # These account for different ways to call the same thing
    $page  =~ s/%7E/~/goi;      # decode tildes ~
    $page  =~ s/\?.*$//o;   # removes ? and parameters from script calls
    $page  =~ s/index.html$//o;   # removes index.html and shortens it to directory
    $page  =~ s/index.htm$//o;    # removes index.htm and shortens it to directory
    $page  =~ s/\#.*$//o;   # removes # internal links and counts same as page
    $page  =~ s|/+|/|go;    # turns multiple slashes into one, not just at end
    $page  =~ s/\`//g;      # removes ` backquotes
    $page =~ s/\/$//; # TODO
   
    $site="\L$site";    # force all the following characters to lower case
    ###$totallines++;

    # 2xx=success, 3xx=redirection, 4xx & 5xx=error
    ###$statuscodes{$status}++;
    next unless (($status=~ /^2/) || ($status eq '304'));

    # get all the time info
    ($date,$month,$year) = 
         ($when =~ m|(..)/(...)/..(..):..:..:..|o);
    ###($date,$month,$year,$hour,$min,$sec) = 
    ###    ($when =~ m|(..)/(...)/..(..):(..):(..):(..)|o);
    $month = $MONTHS{$month};

    # update byte count to take into account images
    $datestring = $year . $month . $date;
    $bytes{$datestring}+=$bytes;
    ###$totalbytes+=$bytes;
    ###$totalhits{$datestring}++;

# This section increments out hit counts for just our pages, and skips the rest
    print "page --> $page\n" if($verbose > 1);
    if (defined($classes{$page})) {
      print "page accepted --> $page [$classes{$page}]\n" if($verbose > 1);
      $classes{$page}++;
    } else {
      next LOGLOOP;
    }

    # Get the domain, if there is one.
    if ($site =~ /\.(com|net|edu|gov|mil|org)$/o) {
      ($domain) = ($site =~ /([^\.]+\.[^\d\.]+)$/o);
    } elsif ($site =~ /\.\D\D$/o) {
      ($domain) = ($site =~ /([^\.]+\.[^\.]+\.\D\D)$/o);
    } else {
      $domain = "Unresolved IP Address";
    }

    # Increment the totals
    $monthstring="${year}${month}";
    ###$total++;
    ###$dates{$datestring}{$page}++;
    ###$pagebytes{$page}+=$bytes;
    $domains{$domain}{$monthstring}++;
    $domains{$domain}{'total'}++;
    ###$hours{$hour}++;
    unless ($user eq '-') {
      $users{$user}{total}++;
      $users{$user}{$monthstring}++;
      }
    }

  close(LOG);
  print "\nDone Reading $LOG\n\n" if $verbose;
  
  $logtime = `date`;
  print OUT "LOG PROCESSING FINISHED AT $logtime";
  print OUT "FIRSTLINE:$firstline{$LOG}";
  print OUT "LASTLINE:$lastline{$LOG}";

  foreach $srs (sort keys %srstable) {
    $correcthits = $classes{$srstable{$srs}};
#   print "srstable{$srs} --> $correcthits\n";  # mike 11-8-98
    print OUT "$srstable{$srs} $correcthits\n";

    next if($correcthits == 0);

    $cmd = "SELECT * FROM stats WHERE srs='$srs' && term='$quarter'";
    $sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
    $rv = $sth->execute || die "Can't execute query: $sth->errstr\n";
    if(defined((@array = $sth->fetchrow_array)) && $#array >= 0){
      $cmd = "UPDATE stats SET hits=$correcthits WHERE srs='$srs' && term='$quarter'";
    } else {
      $cmd = "INSERT INTO stats (term,srs,hits) VALUES ('$quarter','$srs',$correcthits)";
    }

	 print "cmd --> $cmd\n" if($verbose > 1);
    $sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
    $rv = $sth->execute || die "Can't execute query: $sth->errstr\n";
  } 
  close(OUT);
}

sub get_url_list
{
  local($quarter) = $_[0];
  local(@urls, $dir);

  #extract all URLs in the REGurl table
  #$cmd = "select srs,url from REGurl where term='$quarter'";
  $cmd = "select srs,url from REGclasses where term='$quarter'";
  $sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
  $rv = $sth->execute || die "Can't execute query: $sth->errstr\n";

  while(($srs, $url) = $sth->fetchrow_array){
    next unless($url =~ /^$SERVER_NAME/i);
    $url .= '/' unless($url =~ /\/$/); 
    $url =~ /^$SERVER_NAME(.*)\/$/i;
    $dir = $1;
    $srstable{$srs} = $dir;
    push(@urls, $url);
  }
  
  #extract all URLs in the indirect_url table
  $cmd = "select srs,url from indirect_url where term='$quarter'";
  $sth = $dbh->prepare($cmd) || die "Can't prepare $cmd: $dbh->errstr\n";
  $rv = $sth->execute || die "Can't execute query: $sth->errstr\n";
  while(($srs, $url) = $sth->fetchrow_array){
    next unless($url =~ /^$SERVER_NAME/i);
    $url .= '/' unless($url =~ /\/$/); 
    $url =~ /^$SERVER_NAME(.*)\/$/i;
    $dir = $1;
    $srstable{$srs} = $dir;
    push(@urls, $url);
  }

  return @urls;
}
