######################################
####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
# Name: placelink.pm
# Usage: &Place_link("linkurl","linkname")
#   called by upload.pm and linkupload.pm
# Steps: +placelink.pm
#          +Place_link()
#            -modifies coursemenu.inc for particular class
#
# Update 07-22-00 mike  replaced hard-coded with GLOBAL vars
# Update 11-13-98 - changed to allow cols=10 max.
# Update 08-07-98 - changed to point to coursemenu.inc 
# Update 04-04-98 - commented out chgrp debug.txt by chmod g+s, 
#   it also was showing up in error logs
# Update 10-17-97 - rewritten to deal with formatting of cells 
# Update 10-13-97 - added check to make sure a value for $table exists
# Update 07-29-97 - Mike shrunk it down a bit. But it still doesn't put 
#   terminating /TR unless there's a 3rd cell in a row.
# Update - written by Mike Franks & Artin Rebekale
#
# Status: production
# Purpose: this sub simply gets the variable link-name and trys
#   to place it on the web page


sub Place_link {

  my($linkurl,$linkname) = @_;
# my $debug = "YES"; # uncomment this to turn on debugging
  my $table = "";
  
  if ($debug) {
    open(DEBUG,">$basedir/debug.txt") || 
      &error_message("Couldn't open $basedir/debug.txt");
  }

  if (!$table) {
    $table="$basedir/coursemenu.inc";
  }
  system "cp $table $table.old";   # $table comes from calling programs
  chmod (0660, "$table.old");      #readable, writeable, not executable to user/group

  if ($?) {
    &error_message("cp $table $table.old failed");
  }
# print "Content-type: text/html\n\n";
  undef $/;

### mike debugging 7/24/00 The regex isn't finding anything 
  open (INDEX,"+>>$table") || &error_message("open $table failed");
# open (INDEX,"<$table") || &error_message("open $table failed");
  flock INDEX, 2;    # lock file open
  seek INDEX, 0, 0; # on Linux, need this to move pointer back to start of file
  my $cnt = 1;
  $index=<INDEX>;    # suck whole file into $index
  $/ = "\n";
# print "***$index***\n";
  if ($index =~ /<!--menustart(-COLS=(\d+))?-->(.*)<!--menuend-->/s) {
    print DEBUG "FOUND MENU.\n" if $debug;
    $menu = $3;
    $cols = $2;    # maximum number of columns per row
    unless (($cols > 0) && ($cols <= 10)) { $cols = 4; }   # set to 4 if invalid

    $cellcnt = 1;   # to help grab formatting from first cell only
    $rowcnt = 1;  # should cycle between 1 and 4 (or $cols maximum)
    $newmenu = "";  # will eventually replace existing menu section

    # break menu into cells, ignoring row markers. They'll be put back later
    if ($totalcells = (@cells) = ($menu =~ m#(<TD.*?>.*?</TD>)#sig)) {
      print DEBUG "Total Cells = $totalcells\n" if $debug;
      foreach $mcell (@cells) {
        $cell = $mcell;   # to leave original cell exactly the same
        $cell =~ s/\n//sg;     # drop newlines
        $cell =~ s/>(\s*?)</></sg;     # drop blankspace between markup
        if ($cell =~ m#(<TD.*?>)(.*?)<a href="(.*?)">(.*?)</A></TD>#si) {
          print DEBUG "PARSING CELL\n" if $debug;
          $cellstart = $1;
          $preurl  = $2;
          $url     = $3;
          $labelm   = $4;  # label with HTML markup
          $label = $labelm;
          $label =~ s#<.*?>##sg;  # remove HTML markup
          $label =~ s/^\s*//;     # remove whitespace from front
          $label =~ s/\s*$//;     # remove whitespace from back

          # Adam's nifty recursive algorithm to substitute the bugprone code above
          while ($labelm =~ /^(<.*?>)(.*)(<.*?>)$/) {
            $premarkup = $premarkup . $1;
            $postmarkup = $3 . $postmarkup;
            $labelm = $2;
          } #endwhile

          if ($cellcnt == 1) {   # capture formatting from first cell
            $kcellstart = $cellstart; $kpremarkup = $premarkup;
            $kpostmarkup = $postmarkup;
          }
          print DEBUG "  \$cellstart = $cellstart\n" if $debug;
          print DEBUG "  \$preurl = $preurl#####\n" if $debug;
          print DEBUG "  \$url = $url\n" if $debug;
#         print DEBUG "  Label with HTML Markup = $labelm\n" if $debug;
          print DEBUG "  \$premarkup = $premarkup\n" if $debug;
          print DEBUG "  \$label = $label#####\n" if $debug;
          print DEBUG "  \$postmarkup = $postmarkup\n" if $debug;

          if ($rowcnt < $cols) {
            $newmenu .= "\n    <TR>\n" if $rowcnt == 1; # need <TR> to start row
            $newmenu .= "\n        $mcell\n";
            $rowcnt++;     # increment counter within row
          } else {                 # $rowcnt = $cols, so end of row
            $newmenu .= "\n        $mcell\n";
            $newmenu .= "\n    </TR>\n";            # need </TR> to end row
            $rowcnt = 1;
          }
        $cellcnt++;
        }  #  end of  if ($cell =~ m#(<TD.*?>)(.*?)
        print DEBUG "\n\n" if $debug;
      }
      ### end of writing old cells ###

      ### write new cell ###
      $newcell =  "\n        $kcellstart\n";
      $newcell .= "        <A HREF=\"$linkurl\">\n";
      $newcell .= "        $kpremarkup$linkname$kpostmarkup</A></TD>\n\n";
      if ($rowcnt < $cols) {
        $newmenu .= "\n    <TR>\n" if $rowcnt == 1;  # need <TR> to start row
        $newmenu .= $newcell;
        until ($rowcnt == $cols) {  # fill in with blank cells to keep bg color
          $newmenu .= "\n        $kcellstart\n";
          $newmenu .= "          &nbsp;</A></TD>\n\n";
          $rowcnt++;     # increment counter within row
        }
        $newmenu .= "\n    </TR>\n";         # need </TR> to end row
      } else {                 # $rowcnt = $cols, so end of row
        $newmenu .= "$newcell\n    </TR>\n";         # need </TR> to end row
        $rowcnt = 1;
      }
      ### end of writing new cell ###
    } else {
      &error_message("Couldn't find any cells in menu.");
    }   # end of if ($totalcells = (@cells) = ...
    
    ### replace old menu with $newmenu ###
    $index =~ s/(<!--menustart(-COLS=(\d+))?-->)(.*)(<!--menuend-->)/$1$newmenu$5/si;

    seek INDEX, 0, 0;
    truncate INDEX, 0;  # zero out file
    print INDEX $index;      # rewrite file
    close(INDEX);
    #  could use check for file size here
  } else {
#   &error_message("Couldn't find menu section in $index.htm.");
    &error_message("Couldn't find menu section in $table.");
  }
  if ($debug) {
    close(DEBUG);
    chmod (0660, "$basedir/debug.txt");    #readable, writeable, not executable to user/group
  }
}
1
