#!/usr/local/bin/perl
####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
# Name: setgroups.pl
# Purpose: creates term group files. Should be run as part of daily cron job or whenever your list of instructors is downloaded.
#
# 08-31-00 mike & kenny - revised to create admin and webtechs groups
# 9-30-98 - Mike added check of addadmin table for special class permissions
# 8-7-98 - Artin wrote this
#        - Mike altered it to lowercase coursenum before writing, to better
#          match ClassWeb directories.

## $BASE is set in Apache Environment with SetEnv and PassEnv in httpd.conf
use File::Basename;
$BASE = "/home/classweb";
$BASE =~ s/\/security//;
require "$BASE/customize.pl";  # important vars set here
require "$BASE/dbvars.pl";     # database vars set here
require "$BASE/dbopen.pl";
require "$BASE/dbfuncs.pm";

# declared variable here so we can eliminate location.pm
$master_pwd=$PASSFILE;
$student_new="$PASSTUDENT" . ".new";
$student_old="$PASSTUDENT" . ".old";

$term="00S";
$count=2001; #assignment of password
foreach $option (@ARGV) {
  if ($option =~/ta/i) {$do_tas="yes";}
}

#-- flow
&check_previous;
system "touch $PASSTUDENT";
&control;
# &create_password_file;

#-- subs
sub control {
  $cmd5 = "select distinct term from REGclasses";
  $sth5 = $dbh->prepare($cmd5) or die "Cant prepare $cmd: $dbh->errstr\n";
  $rv5 = $sth5->execute or die "Cant execute query: $sth->errstr\n";
  
  open (SPN,">>$student_new") || die "   ***Cannot open studentpass.new, $!\n";
  while ($terms= $sth5->fetchrow_hashref) {
    $term=$terms->{term}; 
    print "Setting up $term...\n";
    open (TG,">$PASSDIR/$term.group") || die "   ***Cannot open $term.group, $!\n";
    open (SG,">$PASSDIR/$term.student-group") || die "   ***Cannot open $term.student-group, $!\n";
    open (CG,">$PASSDIR/classweb.group") || die "   ***Cannot open classweb.group, $!\n";
    &set_admin; # create admin group that can administer all classes
    &set_webtechs; # create webtechs group that can administer all classes
    &set_depts; # create dept groups that can administer their dept's classes
    &set_profs; # create group for each class giving profs admin rights
    &set_students; # create student-group for each class giving students access to wwwboard and grades
    close(TG);
    close(SG);
    close(CG);
    &set_closed; # find classes running sub-terms and CLOSE them if their start day is today
  } 
  close(SPN);
  # quickly put the new password file in place...
  system "mv $PASSTUDENT $student_old";
  system "sort -u $student_new -o $PASSTUDENT && rm $student_new";
  chmod (0660, "$PASSTUDENT");      #readable, writeable, not executable to user/group
}

sub set_closed {
  $cmdA = "select term from current";
  $sthA = $dbh->prepare($cmdA) or die "Cant prepare $cmdA: $dbh->errstr\n";
  $rvA = $sthA->execute or die "Cant execute query: $sthA->errstr\n";
  $currentterm=$sthA->fetchrow_hashref;
  $theterm=$currentterm->{term};
  
  if ("$term" eq "$theterm") {
    # cycle through all the classes and set them closed if open and subterm start is today.
    # someday, make this work for full-term classes when we know the term start day for sure.
    my ($sec,$min,$hour,$mday,$mon,$year) = (localtime(time))[0,1,2,3,4,5];
    @months = ("January","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
    $today="$months[$mon] $mday";
    $cmdB = "select REGclasses.srs from REGmeetings,REGclasses where REGclasses.term='$term' && 
      REGclasses.status='Open' && REGmeetings.startday='$today' && REGmeetings.srs=REGclasses.srs && 
      REGmeetings.term=REGclasses.term";
    $sthB = $dbh->prepare($cmdB) or die "Cant prepare $cmdB: $dbh->errstr\n";
    $rvB = $sthB->execute or die "Cant execute query: $sthB->errstr\n";
    while($somesrs = $sthB->fetchrow_hashref) {
      $thesrs=$somesrs->{srs};
      print "Class srs $thesrs starts session today and is being updated as CLOSED\n";
      $cmdC = "update REGclasses set status='Closed' where term='$term' && srs='$thesrs'";
      $sthC = $dbh->prepare($cmdC) or die "Cant prepare $cmdC: $dbh->errstr\n";
      $rvC = $sthC->execute or die "Cant execute query: $sthC->errstr\n";
    }
  }
}

sub set_admin {
  print TG "admin:"; 
  print CG "admin:"; 
  print "Creating admin group ...\n";
  $cmd3 = "select uid from REGcontacts where dept='admin'";
  $sth3 = $dbh->prepare($cmd3) or die "Cant prepare $cmd: $dbh->errstr\n";
  $rv3 = $sth3->execute or die "Cant execute query: $sth->errstr\n";
  while($contacts = $sth3->fetchrow_hashref) {
    $tuid=$contacts->{uid};
    print TG "$tuid ";
    print CG "$tuid ";
  }
  print TG "\n"; 
  print CG "\n"; 
}

sub set_webtechs {
  print TG "webtechs:"; 
  print CG "webtechs:"; 
  print "Creating webtechs group ...\n";
  $cmd7 = "select uid from REGcontacts where dept !='admin' && type='WebTech'";
  $sth7 = $dbh->prepare($cmd7) or die "Can not prepare $cmd: $dbh->errstr\n";
  $rv7 = $sth7->execute or die "Can not execute query: $sth->errstr\n";
  while($contacts = $sth7->fetchrow_hashref) {
    $tuid=$contacts->{uid};
    print TG "$tuid ";
    print CG "$tuid ";
  }
  print TG "\n"; 
  print CG "\n"; 
}

sub set_depts {
  print "Creating Department Groups...\n";
  $cmd = "select dept,dirname from REGdept ";
  $sth = $dbh->prepare($cmd) or die "Can not prepare $cmd: $dbh->errstr\n";
  $rv = $sth->execute or die "Can not execute query: $sth->errstr\n";
  while ($depts= $sth->fetchrow_hashref) {
    print TG "$depts->{dirname}:"; 
    $temp=$depts->{dept};
    $temp2=$depts->{dirname};
    $dept_list{$temp}=$temp2;
    $cmd3 = "select uid from REGcontacts where dept='$temp'";
    $sth3 = $dbh->prepare($cmd3) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv3 = $sth3->execute or die "Cant execute query: $sth->errstr\n";
    unless ($depts->{dirname} =~ /admin/) {
      while ($contacts=$sth3->fetchrow_hashref) {
        $tuid=$contacts->{uid};
        print TG "$tuid ";
        $prof_u{$tuid}=$count if (!$prof_u{$tuid} && $tuid); 
        $count+=17;
      }
    }
  print TG "\n";
  }
}

sub set_profs {
  print "Creating Class Groups...\n";
  $cmd = "select srs,coursenum,dept,section,secidx from REGclasses where term='$term'";
  $sth = $dbh->prepare($cmd) or die "Cant prepare $cmd: $dbh->errstr\n";
  $rv = $sth->execute or die "Cant execute query: $sth->errstr\n";
 
  while ($classes= $sth->fetchrow_hashref) {
    $secidx=$classes->{secidx};
    $coursenum=$classes->{coursenum}; 
    $lcoursenum=lc($coursenum);  # need lowercase of coursenum
    $temp=$classes->{dept};
    print TG "$dept_list{$temp}$lcoursenum-$classes->{section}:"; 
    $srs=$classes->{srs}; 
    $cmd2= "select uid from REGprof where srs='$srs' && term='$term'";
    $sth2 = $dbh->prepare($cmd2) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv2 = $sth2->execute or die "Cant execute query: $sth->errstr\n";

    while ($tprofs= $sth2->fetchrow_hashref) {
      print TG "$tprofs->{uid} ";
      $tuid=$tprofs->{uid};
      $prof_u{$tuid}=$count if (!$prof_u{$tuid} && $tuid); 
      $count+=17;
    } 
    if ($do_tas) {
      $cmd4= "select uid from REGta where term='$term' && dept='$temp' && relsecidx='$secidx' && coursenum='$coursenum'";
      $sth4 = $dbh->prepare($cmd4) or die "Cant prepare $cmd: $dbh->errstr\n";
      $rv4 = $sth4->execute or die "Cant execute query: $sth->errstr\n";

      while ($tprofs= $sth4->fetchrow_hashref) {
        print TG "$tprofs->{uid} ";
        $tuid=$tprofs->{uid};
        $prof_u{$tuid}=$count if (!$prof_u{$tuid} && $tuid); 
        $count+=17;
      } 
    } # end of do_tas

    # Check for any extra from addadmin table
    $cmd6= "select uid from addadmin where term='$term' && srs='$srs'";
    $sth6 = $dbh->prepare($cmd6) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv6 = $sth6->execute or die "Cant execute query: $sth6->errstr\n";

    while ($tprofs= $sth6->fetchrow_hashref) {
      print TG "$tprofs->{uid} ";
      $tuid=$tprofs->{uid};
      $prof_u{$tuid}=$count if (!$prof_u{$tuid} && $tuid); 
      $count+=17;
    } 
    print TG "\n"; 
  }
}

sub set_students {
  print "Creating Student Groups...\n";
  $cmd = "select srs,coursenum,dept,section,secidx from REGclasses where term='$term'";
  $sth = $dbh->prepare($cmd) or die "Cant prepare $cmd: $dbh->errstr\n";
  $rv = $sth->execute or die "Cant execute query: $sth->errstr\n";
 
  while ($classes= $sth->fetchrow_hashref) {
    $secidx=$classes->{secidx};
    $coursenum=$classes->{coursenum}; 
    $lcoursenum=lc($coursenum);  # need lowercase of coursenum
    $temp=$classes->{dept};
    print SG "$dept_list{$temp}$lcoursenum-$classes->{section}:"; 
    $srs=$classes->{srs}; 
    
    # add students...
    $cmd2= "select uid from REGrosters where srs='$srs' && term='$term'";
    $sth2 = $dbh->prepare($cmd2) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv2 = $sth2->execute or die "Cant execute query: $sth->errstr\n";
    # get enrollment statuses
    $cmd3= "select enroll_status from REGrosters where srs='$srs' && term='$term'";
    $sth3 = $dbh->prepare($cmd3) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv3 = $sth3->execute or die "Cant execute query: $sth->errstr\n";

    while ($students= $sth2->fetchrow_hashref) {
      $enrolled= $sth3->fetchrow_hashref;
      if($enrolled->{enroll_status} eq "E"){
        print SG "$students->{uid} ";
        $suid=$students->{uid};
        $student_u{$suid}=$count if (!$student_u{$suid} && $suid); 
        $count+=17;

        # write student password file - add entry if not already there (from another class)
        open (SP,"$PASSTUDENT") || die "   ***Cannot open studentpass, $!\n";
        @lines = <SP>;
	close(SP);
        $match = "";
        foreach $line(@lines){
          $uuid = (split(/:/, $line))[0];
          if($uuid eq $suid) { #the student already has a password
            chomp $line;
	    $match = $line;
            last;
          }
        }
        if($match){
          print SPN "$match\n";
        } else {
          $passwd="cw" . int ((rand)*10000000);
          system"$HTPASSWD -b $student_new $suid $passwd";
	  
          $cmd9= "select email from REGrosters where srs='$srs' && term='$term' && uid='$suid'";
          $sth9 = $dbh->prepare($cmd9) or die "Cant prepare $cmd: $dbh->errstr\n";
          $rv9 = $sth9->execute or die "Cant execute query: $sth->errstr\n";
          $emailhash = $sth9->fetchrow_hashref;
	  $email = $emailhash->{email};
	  if ($email){
	  
	    $body="Hello!\n
You have just been added to the roster of at least one class for the year '$term term
at $DIVISION_NAME.  As an enrolled student, you now have access to the 
Classweb system on the web at http://$SERVER_NAME$ENTRY.\n
Classweb offers immediate access to class syllabus, files, grades, discussion 
forum(s), instructor contact info, announcements, relevant web links, and more.\n
Access to grades and authenticated discussion forums requires a Student UID# and
personal password.
            Your Student UID# is \"$suid\"
	    Your personal password is: \"$passwd\"\n
Please, never give your password out to anyone since all their actions will be
traced to you.\n
Access to the Private Files in each Classweb class area also uses your Student
UID# (above), but is controled by a password that your instructor will assign
in class.\n
If this message has reached you in error, please forward it to 
$REGISTRAR by simply replying to this message.\n
Thank you!";

	    $msg = "From: $REGISTRAR\n";
  	    $msg .= "To: $email\n";
  	    $msg .= "Subject: Classweb Password\n\n";
  	    $msg .= $body;

  	    open(MAIL,"| $SEND_MAIL_CMD");
  	    print MAIL "$msg";
  	    close(MAIL);
	  }
        }
      }
    }
    
    # add TA's...
    if ($do_tas) {
      $cmd4= "select uid from REGta where term='$term' && dept='$temp' && relsecidx='$secidx' && coursenum='$coursenum'";
      $sth4 = $dbh->prepare($cmd4) or die "Cant prepare $cmd: $dbh->errstr\n";
      $rv4 = $sth4->execute or die "Cant execute query: $sth->errstr\n";

      while ($students= $sth4->fetchrow_hashref) {
        print SG "$students->{uid} ";
        $suid=$students->{uid};
        $student_u{$suid}=$count if (!$student_u{$suid} && $suid); 
        $count+=17;

        # write their password & uid to student password file
        open (SP,"$PASSFILE") || die "   ***Cannot open classpass, $!\n";
        @lines = <SP>;
	close(SP);
        foreach $line(@lines){
          $uuid = (split(/:/, $line))[0];
          if($uuid eq $suid) { #the student already has a password
            print SPN "$line";
            last;
          }
        }
      } 
    } # end of do_tas

    # add faculty from addadmin table...
    $cmd6= "select uid from addadmin where term='$term' && srs='$srs'";
    $sth6 = $dbh->prepare($cmd6) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv6 = $sth6->execute or die "Cant execute query: $sth6->errstr\n";

    while ($students= $sth6->fetchrow_hashref) {
      print SG "$students->{uid} ";
      $suid=$students->{uid};
      $student_u{$suid}=$count if (!$student_u{$suid} && $suid); 
      $count+=17;

      # write their password & uid to student password file
      open (SP,"$PASSFILE") || die "   ***Cannot open classpass, $!\n";
      @lines = <SP>;
      close(SP);
      foreach $line(@lines){
        $uuid = (split(/:/, $line))[0];
        if($uuid eq $suid) { #the student already has a password
          print SPN "$line";
          last;
        }
      }
    } 

    # add class prof(s)...
    $cmd8= "select uid from REGprof where term='$term' && srs='$srs'";
    $sth8 = $dbh->prepare($cmd8) or die "Cant prepare $cmd: $dbh->errstr\n";
    $rv8 = $sth8->execute or die "Cant execute query: $sth8->errstr\n";

    while ($students= $sth8->fetchrow_hashref) {
      print SG "$students->{uid} ";
      $suid=$students->{uid};
      $student_u{$suid}=$count if (!$student_u{$suid} && $suid); 
      $count+=17;

      # write their password & uid to student password file
      open (SP,"$PASSFILE") || die "   ***Cannot open classpass, $!\n";
      @lines = <SP>;
      close(SP);
      foreach $line(@lines){
        $uuid = (split(/:/, $line))[0];
        if($uuid eq $suid) { #the student already has a password
          print SPN "$line";
          last;
        }
      }
    } 
    
    print SG "\n"; 
  }
}

sub check_previous {
  if (-s $master_pwd) { 
    print "Previous Password File Found\n";
    open(MASTER,"<$master_pwd"); 
    @master=<MASTER>;
    close(MASTER); 
    foreach $master (@master) {
      $master=~/(.*):(.*)/;
      $profs{$1}=$2;
    }
  }
}
