#!/usr/local/bin/perl 
####################################################################
# ClassWeb - Copyright (C) 2000 UC Regents & UCLA Social Sciences  #
# Computing. See http://classweb.ucla.edu/license.htm for license. #
####################################################################
# Name: setup.pl  
# Usage: setup.pl term srs# for single class setup or 
#   setup.pl term batch for batch mode setup 
# Status: production
# Purpose: sets up one or more ClassWeb sites (single or batch mode)
#  - should be run as web server ID
# Needs: 
# Update: 09-03-02 Jon - added contenttype.inc require line for multilingual content-type
# Update: 08-22-00 Nick and Jerome - Substituting out language-dependent text
#   strings with variables to allow multilingual support.
# Update: 09-06-00 kenny - automatically runs before single or batch setup
# Update: 08-30-00 kenny - continue'n to clean up and tweak
# Update: 07-28-00 mike - give error when dept not in REGdept table, with
# instructions for adding new depts before creating classweb sites.
# Update: 07-19-00 mike - added -p param to system (cp to preserve rights
#     - now getting $BASE from environment variable. This
# works when Perl script is called via web, but not when called via PHP script.
# In that case, it needs to be put into the environment again by the PHP script.
#        - classweb_undo now clears out test URLs before setting up batch
# Update: 07-15-00 mike - removed password section
# Update: 07-14-00 kenny & mike - debugging for test RedHat Linux box
#  
# Update: 07-11-00 kenny - abstracting; transfering UCLA specific
#                          vars to and setting them in customize.inc
# Update: 08-03-99 mike - changed wwwadmin.pl from template to payload copy 

## $BASE is set in Apache Environment with SetEnv and PassEnv in httpd.conf
$BASE = "/home/classweb"; # main classweb programs directory

# if setup.pl is run from a cronjob or the command line
# following code is necessary to determine the BASE prog. dir
unless ($BASE) { # If $BASE doesn't exist, use calling path or current directory
  use File::Basename;
  $BASE = dirname($0); # set $BASE to this program's calling path
  unless ($BASE) {
    # for programs called from this directory via telnet command with . on path
    $BASE=".";  # set to current directory if it still doesn't exist
  }
}

  require "$BASE/customize.pl";  # important vars set here
  require "$BASE/language.pl";  ###Multilingual support
  require "$BASE/dbvars.pl";     # database vars set here
  require("$BASE/dbopen.pl");

#######################################################
#$DEBUG = "DEBUGGING: "; # uncomment while debugging 
$DEBUGGER = $EMAIL_OF_CLASSWEB_ADMIN; 

$PROFBASE = $BASE . "/prof"; # main classweb models location
chdir "$PROFBASE" || die "Can't cd to $PROFBASE: $!\n";


####################################################### 
# needed to update classweb data in the iei database
# this is UCLA's registrar's database; not necessary
# require "$BASE/urlupdate.pm"; 

# needed to add an entry into the board table when a new bb is set up
require "$BASE/update_boards.pm";
                         
#######################################################
# IEI is an acronym for Instructional Enhancement Initiative.
# It's a technology fee at UCLA imposed on undergrad students to
# fund building class websites and creating new computer labs. 
#
# # tells record's office who is making updates to the campus db
# $IEIposter = $ADMIN_NAME; 
#
# # tells record's office who is making updates to the campus db
# $IEIemail = $EMAIL_OF_CLASSWEB_ADMIN;
########################################################

$default_sender = $DEFAULT_SENDER;  
$NOTIFY = $EMAIL_OF_CLASSWEB_ADMIN;
$classbase = "$DOCUMENT_ROOT$CLASSES";  

#######################################################
# sets up custom vars for depts that have      
# individual dept webtechs who need ftp access to either
# normal or custom ClassWeb groups.  the hash sets up
# custom vars for depts that are applicable.

# Unix group owner to be assigned for class sites in these depts. Default group
# ID is 'classweb'. These left as an example.
%customgroup = (
#  HIST => 'history',
#  ECON => 'ieiecon',
#  ANTHRO => 'ieianth',
#  GEOG   => 'ieigeog',
#  SOCIOL => 'ieisoc',
#  'POL SCI' => 'ieipoli'
);

# location of custom templates for depts that prefer their own templates
%customtemplatedir = (
# HIST => '/info-data/www-data/history/routines/models4'
);

# setup directory for custom templates
%customsetup = (
# HIST => '/info-data/www-data/history/routines/models4/setup'
);

# specific person(s)'s e-mail that setup messages are sent from
%customsender = (
# HIST => 'Brenda Stevenson <historyweb@history.ucla.edu>'
);

# specific person(s) who gets notified with specific template setups
%customnotify = (
# HIST => 'historyweb@history.ucla.edu'
);

$|++;     #  unbuffers output   - maybe better in calling routine


####################################################
# handle command line parameters
$term = $ARGV[0];
$targetsrs = $ARGV[1];
$batchmode = 1 if $targetsrs =~ /batch/i;   

####################################################
# load exceptions file of drops and adds

  open(EXCEPT,"$BASE/exceptions.dat") || die("Couldn't open file exceptions.dat.");
  while(<EXCEPT>) {
    if (/^(\d\d\w),(\d{9})/ ) {
      $termsrs = $1 . $2;
      $drops{$termsrs}++;  # classes to drop from website setup
    }
  }
  close(EXCEPT);

####################################################
# if batch mode, start working through master datafile of courses 
# by collecting URLs for this quarter all at once, to save time later

if ($batchmode) {
  $cmd = "select term,srs,url from REGclasses where term='$term'";
  #$cmd = "select * from REGurl where term='$term'";
  $sth = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
  $sth->execute or die "Can't execute query: $sth->errstr\n";
  while($classurl = $sth->fetchrow_hashref) {
    $url{$classurl->{srs}} = $classurl->{url};
  }
}

# update group authen security files with DB data
`$PASSDIR/setgroups.pl`;

####################################################
# if single course mode, just go direct to term, srs query

if (!$batchmode) {
####################################################
# single class setup
####################################################
  $cmd = "select * from REGclasses where term='$term' && srs = '$targetsrs'";

  $sth = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
  $sth->execute or die "Can't execute query: $sth->errstr\n";

  if ($classes = $sth->fetchrow_hashref) {
    die("Class cancelled\n") if ($classes->{status} eq "Cancelled");
    $srs=$targetsrs;
    &setup($term,$targetsrs);

################################################################
# code used to update status in UCLA's IEI database
################################################################
#
# temporarily exit before updating anything
# exit("Temporarily quitting before URL updates."); #uncomment while debugging
#
# send to Eric Splaver's database, using Chris Spreitzer's form
# if (&urlupdate($term,$srs,$classurl,$IEIposter,$IEIemail,$errmsg)) {
#   print " IEI UPDATE OK: $errmsg\n";

# records status of site in sitelog; IEI status info removed
$cmdlog = "replace into sitelog (term, srs, url, dir, status, updateruid) values ('$term','$srs','$classurl','$outdir','$DEBUG CREATED','$UPDATER_ID')";  

#   $cmdlog = "replace into sitelog (term, srs, url, dir, status, updateruid) values ('$term','$srs','$classurl','$outdir','$DEBUG CREATED: IEI Update OK','601270210')"; # CUSTOMIZE - enter UID no. here: i.e. 601270210 
# } else {
#   print " IEI UPDATE FAILED: $errmsg\n";
#   $cmdlog = "replace into sitelog (term, srs, url, dir, status, updateruid) values ('$term','$srs','$classurl','$outdir','$DEBUG CREATED: IEI Update Failed: $errmsg','601270210')"; # CUSTOMIZE - enter UID no. here
# }
#################################################################

    # update sitelog table to keep record
    $sthlog = $dbh->prepare($cmdlog) or die "Can't prepare $cmdlog: $dbh->errstr\n";
    $sthlog->execute or die "Can't execute query: $cmdlog: $sthlog->errstr\n";
    $sthlog->rows;

    # update REGurl for our immediate use
    #$cmdurl = "replace into REGurl (term,srs,url) values ('$term','$srs','$classurl')";
    $cmdurl = "update REGclasses set url='$classurl' where term='$term' and srs='$srs'";
    $sturlh = $dbh->prepare($cmdurl) or die "Can't prepare $cmdurl: $dbh->errstr\n";
    $sturlh->execute or die "Can't execute query: $sturlh->errstr\n";
  } else {
    die("No results: $cmd\n");
  }
} else {
####################################################
# batch mode setup
####################################################
  $testcnt = 1;

# the list of exceptions for classes that should not be set up are included 
# in this section. when running in batch mode setup, the exceptions need
# to be added to the list in this section. 

## CUSTOMIZE 
  $cmd = "select * from REGclasses where term='$term' && 
    status <> 'Cancelled' && 
    crsidx < 199 && 
    dept <> 'DEMO' &&
    dept <> 'GE CLST' && 
    dept <> 'HNRS' && 
    dept <> 'ENVIRON'"; 
  $sth = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
  $sth->execute or die "Can't execute query: $sth->errstr\n";
  while($classes = $sth->fetchrow_hashref) {
    next if $url{$classes->{srs}};  # skip those with URLs
    $srs = $classes->{srs};
    if ($drops{$term . $srs}) {

# print "SKIPPING, on drop list: $term $classes->{dept} $classes->{coursenum} $classes->{section} $srs\n"; # uncomment while debugging the drop list 
     next;
   }

## CUSTOMIZE 
# special cases not to be included are noted in this section below
#skip POL SCI 194s, POL SCI 195B, POL SCI 195C, INT DEV 195s, ROTC Drill class
#   next if (($classes->{dept} eq "POL SCI") 
#     && ($classes->{coursenum} eq "194"));  # skip POL SCI 194s
#   ## 10-28-99 PS 195A is ok, but B and C are not
#   next if (($classes->{dept} eq "POL SCI") 
#     && ($classes->{coursenum} =~ /^195B/));  # skip POL SCI 195B
#   next if (($classes->{dept} eq "POL SCI") 
#     && ($classes->{coursenum} =~ /^195C/));  # skip POL SCI 195C
#   next if (($classes->{dept} eq "INT DEV") 
#     && ($classes->{coursenum} =~ /^195/));  # skip INT DEV 195s per bug #34
#   next if (($classes->{dept} =~ /(NAVAL|AERO ST|MIL SCI)/) 
#     && ($classes->{coursenum} =~ /^Z/));    # skip ROTC Drill classes

    print "#$testcnt ";
    print "$classes->{term}  $classes->{dept}  $classes->{coursenum} $classes->{section} $srs ";
    $testcnt++;
# $testing = "1"; # uncomment while deubugging 
if ($testing) {  
  print " \n";
} else {
    &setup;  # need error check here

######################################################
# more code to update UCLA IEI database; not necessary
######################################################
# send to Eric Splaver's database, using Chris Spreitzer's form
# if (&urlupdate($term,$srs,$classurl,$IEIposter,$IEIemail,$errmsg)) {
#   print " IEI UPDATE OK: $errmsg\n";
#   $cmdlog = "replace into sitelog (term, srs, url, dir, status, updateruid) values ('$term','$srs','$classurl','$outdir','CREATED: IEI Update OK','$UPDATER_ID')";
# } else {
#   print " IEI UPDATE FAILED: $errmsg\n";
#   $cmdlog = "replace into sitelog (term, srs, url, dir, status, updateruid) values ('$term','$srs','$classurl','$outdir','CREATED: IEI Update Failed: $errmsg','$UPDATER_ID')";
# }
####################################################

######################################
# update sitelog table to keep record
    $sthlog = $dbh->prepare($cmdlog) or die "Can't prepare $cmdlog: $dbh->errstr\n";
    $sthlog->execute or die "Can't execute query: $cmdlog: $sthlog->errstr\n";
    $rv = $sthlog->rows;
# print "MySQL sitelog: $rv rows updated.\n"; # 4-10-00 aimee - can remove 

#######################################
# update REGurl for our immediate use
    #$cmdurl = "replace into REGurl (term,srs,url) values ('$term','$srs','$classurl')";
    $cmdurl = "update REGclasses set url='$classurl' where term='$term' and srs='$srs'";

    $sturlh = $dbh->prepare($cmdurl) or die "Can't prepare $cmdurl: $dbh->errstr\n";
    $sturlh->execute or die "Can't execute query: $sturlh->errstr\n";
    $rv = $sturlh->rows;
# print "MySQL REGurl: $rv rows updated.\n"; # 4-10-00 aimee - can remove
  }
}
}  # end of batch mode setup

################
# Subroutines  #
################

####################################################
# CUSTOMIZE - our registrar's listing is unique and this section
# will need to be customized to accomodate your local record's
# office database listings
sub setup {
  # called by single class setup and batch mode setup
  # REGclasses row available in $classes->{field_name} array
  # e.g. $classes->{term}, $classes->{dept}, etc.
  #
  # $term and $srs are available globally
  $dept = $classes->{dept};
  $num = $classes->{coursenum};
  $sect = $classes->{section};

####################################################
  
# get dept dirname or report error message
  $cmd = "select * from REGdept where dept='$dept'";
  $sthdept = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
  $sthdept->execute or die "Can't execute query: $sthdept->errstr\n";
  if ($REGdept = $sthdept->fetchrow_hashref) {
    $deptdir = $REGdept->{dirname};
  } else {
    &error("No dirname found in REGdept table for DEPT=$dept. Please add an entry to that table in order to be able to create class websites.\n");
    # die("No dirname found in REGdept table for DEPT=$dept\n");
  }

  %FORM = ();    # make sure nothing carries over
  %FORM = ('term' => "$term",
           'srs'  => "$srs",
          );

  &get_date;  # retrieve date

  $curl = &write_page; # create directory, write pages and setup bulletin board

  &send_email($term, $srs, $dept, $num, $sect, $curl);
  return;
  # closing brace is at bottom so that rest of subroutines are included

sub send_email
{
  # called by setup routine
  local($term, $srs, $dept, $coursenum, $sect, $url) = @_;
  local($cmd, $sth, $sth2, $uid, *MAIL);
  local($be, $num) = (0,0); # has anyone been e-mailed?
  local($email, $num, $to) = "";
  local(%hash); #used to keep info about profs e-mail addresses

  # get list of profs teaching this class
  $cmd = "select uid,reg_name from REGprof where term='$term' && srs='$srs'";
  $sth = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
  $sth->execute or die "Can't execute query: $sth->errstr\n";
  while(@res = $sth->fetchrow_array) {
    $uid = $res[0];
    $num++;
    $cmd = "select email,fullname from profsloc where uid='$uid'";
    $sth2 = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
    $sth2->execute or die "Can't execute query: $sth->errstr\n";
    ($email,$name) = $sth2->fetchrow_array;

# did we find a name for this prof?
    if ($name == '') { #use REGprof name instead
       $name = $res[1];
    } #endif

############################################################
# there are two different locations to check for prof's
# e-mail address - profsloc which is the local location
# for a prof's e-mail and also from the daily telecom dl
# which tends to be outdated. profsloc takes precedence.
############################################################
# profsloc location
    if($email =~ /[^;]+@[^;]+/) {
      $hash{$uid} = "$name" . ";" . $email;
      $be++;
    }

    $cmd = "select email,fullname from REGfaculty where uid='$uid'";
    $sth2 = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
    $sth2->execute or die "Can't execute query: $sth->errstr\n";
    $prevname = $name;
    ($email, $name) = $sth2->fetchrow_array;
    if ($name == '') { # replace name with previous name and
                       # replace values in hash
       $name = $prevname;
       ($testname, $testemail) = split(';', $hash{$uid});
       $hash{$uid} = "$name" . ";" . $testemail;
    } #endif

# profs location
  if($email =~ /[^;]+@[^;]+/) {
    ($testname, $testemail) = split(';', $hash{$uid});
    $be++ if ($testemail !~ /[^;]+@[^;]+/);
      # in other words, increment $be only if there was no valid
      # email before
    $hash{$uid} = "$name" . ";" . $email;
  }

# test for valid email
  ($testname, $testemail) = split(';', $hash{$uid});
  if ($testemail !~ /[^;]+@[^;]+/ ) {
      # email, for one reason or another, is not in correct format
    $hash{$uid} = "$testname" . ";" . "No e-mail address!";
    $to = "";
    $noemail=1;  # flag, 1 means no email found
  } else { #if valid email
    $noemail=0;  # flag, 1 means no email found
    $to = $testemail;
  }   # endif

  $found = 0;

  if ($term =~ /^([0-9]+)1$/) {  #summer session term
    $nameterm = $1 . " Summer Session";
  } else {
    $nameterm = $term;
  }
  open(MAIL,"|$SEND_MAIL_CMD");

# decide who message will be from - custom sender or default sender
  if ($customsender{$dept}) {
    print MAIL "From: $customsender{$dept}\n";
  } else {
    print MAIL "From: $default_sender\n";
  }
  if ($noemail) {  # no email found for prof
    print MAIL "To: $NOTIFY\n" unless $DEBUG;
    print MAIL "To: $DEBUGGER\n" if $DEBUG;
    print MAIL "Bcc: $customnotify{$dept}\n" if $customnotify{$dept};
    print MAIL "Subject: $DEBUG$term $dept $coursenum-$sect ClassWeb site created";
    print MAIL " NOEMAIL\n";
    $to = "NO EMAIL ADDRESS FOUND";
    &log_profcontact($uid,"$DEBUG$term $dept $coursenum-$sect ClassWeb site created: $to","Find email address and forward message.");
  } else {         # email found for prof
    print MAIL "To: $to\n" unless $DEBUG;
    print MAIL "To: $DEBUGGER\n" if $DEBUG;
    print MAIL "Bcc: $NOTIFY\n";
    print MAIL "Bcc: $customnotify{$dept}\n" if $customnotify{$dept};
    print MAIL "Subject: $DEBUG$term $dept $coursenum-$sect ClassWeb site created\n";
    &log_profcontact($uid,"$DEBUG$term $dept $coursenum-$sect ClassWeb site created: sent to $to","");
  } # endif
### insert template code here
## use custom dept setup message if it exists, otherwise use default
  if ($customsetup{$dept} && -e "$customsetup{$dept}/setup.msg.$term") {
    open(MODEL,"<$customsetup{$dept}/setup.msg.$term");
  } elsif (-e "$PROFBASE/setup.msg.$term"){
    open(MODEL,"<$PROFBASE/setup.msg.$term");  
  } else {
    open(MODEL,"<$PROFBASE/setup.msg");
  }

  if(-e $PASSFILE){
    open(FILE, "$PASSFILE");
    @lines = <FILE>;
    $write = 1;
    foreach $line(@lines){
      $uuid = (split(/:/, $line))[0];
      if($uuid eq $uid) { #the user already has a password
        $write = 0;
        last;
      }
    }
    if($write){
      $passwd="cw" . int ((rand)*10000000);
      system"$HTPASSWD -b $PASSFILE $uid $passwd";
    } else {
      $passwd="(use your existing password)";
    }
  }
## replace the #=codes=# in setup.msg with variables from FORM hash
## need to make sure they all exist though by checking the msg

    # setup lastname
    if ($testname && (($FORM{'lastname'}) = ($testname =~ /^(.*,)/))) {
      $FORM{'lastname'} = ucfirst(lc($FORM{'lastname'}));
    } elsif ($testname) {
      $FORM{'lastname'} = $testname;
    } else {
      $FORM{'lastname'}  = "NOT ASSIGNED YET";
    }
    $FORM{'to'}        = $to;
    $FORM{'dept'}      = $dept;
    $FORM{'coursenum'} = $coursenum;
    $FORM{'sect'}      = $sect;
    $FORM{'nameterm'}  = $nameterm;
    $FORM{'url'}       = $url;
    $FORM{'uid'}       = $uid;
    $FORM{'passwd'}    = $passwd;
    $FORM{'webtech'}   = $webtech;

    while (<MODEL>) {
      s/#=(.*?)=#/$FORM{$1}/g;   # variable between #= and =#
      print MAIL;
    }
    close(MODEL);
    close(MAIL);
    if(-e $PASSFILE){
      close(FILE);
    }

  } #endwhile


  if ($num == 0) {
      # no professors
    open(MAIL,"|$SEND_MAIL_CMD");
    print MAIL "To: $NOTIFY\n";
    print MAIL "Subject: $term $dept $coursenum-$sect ClassWeb site created";
    print MAIL " NOPROF\n";
    print MAIL "The site has been setup.\n\n";
    print MAIL "The URL for it is: $url/.\n";
    print MAIL "But no professor was found for this course.\n";
    close (MAIL);
  } # endif
}

####################################################
sub get_date
{
  # called by setup routine
  my ($sec,$min,$hour,$mday,$mon,$year) = (localtime(time))[0,1,2,3,4,5];
  if ($hour < 10) { $hour = "0$hour"; }
  if ($min < 10) { $min = "0$min"; }
  if ($sec < 10) { $sec = "0$sec"; }
  if ($year > 99) { $year = ($year-100); }
  if ($year < 10) { $year = "0$year"; }

  $mon++;  # to go from 0-11 to 1-12
  $date = "$mon-$mday-$year $hour:$min";  # want shorter format for log
}

####################################################
sub write_page {

  # called by setup routine
  $quarter = $term;    # 98W or TEST
  $quarter = $quarter . "X" if ($term =~ /(98S|981)/);  # to avoid overwriting
                                                        # existing quarters

  $payload = "$PROFBASE/payload"; 

  if ($customtemplatedir{$dept}) {
    $FORM{'templatedir'} = $customtemplatedir{$dept};  
  } else {
    $FORM{'templatedir'} = "$PROFBASE/models"; 
  }
  # CUSTOMIZE - class listings might not have same format
  # concatenate $classbase, quarter, dept & classnum to create dir name
  $classdir = lc("$deptdir$num-$sect"); # CUSTOMIZE
  # should now look like econ1-1 or afro-amm117-2 or polisci142-1

  $termdir = "$classbase/$quarter"; 
  $outdir = "$classbase/$quarter/$classdir"; 
  $classurl = "$SERVER_NAME$CLASSES/$quarter/$classdir/"; 

####################################################
## need check for existence of TERM directory, ie: $classbase/$quarter/
## if it doesn't exist, we should have a routine that creates it, with
## the right .htaccess files, permissions, etc.  of course, we also need 
## the header image file for it from the Web Designer as well.
####################################################
 
  unless (-d $termdir) {
    mkdir $termdir, 0775;
    chmod (0770, $termdir);
    system "chmod g+s $termdir";
    system "cp -p $BASE/term_htaccess $termdir/.htaccess";
   
  }

####################################################
# check for overwrite of existing class directory

  &error("directory_exists",
    "The directory $outdir already exists. This means that this Dept and Class
    Number are already being used in the quarter $quarter. Remove it and try
    again.")
     if (-e $outdir);

####################################################
# create class directory in correct quarter directory
####################################################
  mkdir $outdir, 0775;

# we need to chgrp to customgroup value if one exists 
  if ($customgroup{$dept}) {
    system("chgrp $customgroup{$dept} $outdir");  # change group for each dept
  }
  chmod (0770, $outdir);
  system "chmod g+s $outdir";

  unless (-e "$outdir" ) {  # check after creating it
    &error("Couldn't create $outdir",
    "mkdir must have failed.");
  }

####################################################
# copy all files and subdirs in and below as_is subdirectory
# this includes admin and wwwboard directories
  system "cp -rp $payload/* $outdir/";
  system "touch $outdir/announce.inc";

####################################################
# use templates to write out files with variables

  $FORM{'install_date'} = time(); # time is for announcements to compare against
  $FORM{'ENTRY'} = $ENTRY;  
  $FORM{'SCHOOL_COLOR1'} = $SCHOOL_COLOR1;
  $FORM{'SCHOOL_COLOR2'} = $SCHOOL_COLOR2;
  &template("classinfo.inc","classinfo.inc");
  &template("coursemenu.inc","coursemenu.inc"); 

####################################################
# important variables for later

  $dir = $outdir;
  $url = $dir;       # base URL for this class.

# next line converts path to URL but doesn't work unless the path is
# within the Web directory, eg. /web
  $url =~ s#$DOCUMENT_ROOT#http://$SERVER_NAME#;
  $dir .= "/wwwboard";

####################################################
# create discussion board if they want

  chdir $outdir || &error("chdir", "Couldn't chdir to $outdir");
  $masterbbs = "$BASE/wwwboard";

# set up call to update_boards.pm to make entry about new board in boards table
  $directory = "$quarter/$classdir/wwwboard/";
  &update_boards($term, $srs, $directory, "Y", "main", "class") ||
    die("update_boards.pm failed."); 

####################################################
# use templates to write out these pages

  $FORM{'url'} =  "$url";
  &template("admin.fedit-pub.pl","admin/fedit-pub.pl");  # file manager
  &template("admin.fedit-priv.pl","admin/fedit-priv.pl");  # file manager

####################################################
# use templates to write out this page

  $FORM{'posturl'} =  "$url/wwwboard/post.pl";
  $FORM{'sposturl'} = $FORM{'posturl'};
  $FORM{'sposturl'} =~ s/^http:/http:/i;
  &template("auth.inc","wwwboard/auth.inc");  # Authentication control file

####################################################
# create admin .htaccess

  $FORM{'admintitle'} = $dept . " " . $num . ": Website Administration";
  $FORM{'classdir'} = $classdir;
  $FORM{'deptdir'} = $deptdir;
  $FORM{'dirname'} = "wwwboard";
  $FORM{'passfile'} = $PASSFILE;
  $FORM{'passdir'} = $PASSDIR;

  &template("admin.htaccess","admin/.htaccess"); 

####################################################
# create wwwboard .htaccess

  $FORM{'classdir'} = $classdir;
  $FORM{'passtudent'} = $PASSTUDENT;
  $FORM{'passdir'} = $PASSDIR;
  $FORM{'entry'} = $ENTRY;

  &template("wwwboard.htaccess","wwwboard/.htaccess"); 

####################################################
# create private .htaccess

# $FORM{'private_realm'} = $dept . " " . $num . " Private Directory";
  $FORM{'private_realm'} = $dept . " " . $num . " (use SID and URSA Pin)";
  $FORM{'webdir'} = $DOCUMENT_ROOT;
  $FORM{'entry'} = $ENTRY;

  &template("private.htaccess","private/.htaccess"); 

####################################################
# create templated stub.pl -- added by kenny 7.19.00

  $FORM{'progdir'} = $BASE;
  $FORM{'webdir'} = $DOCUMENT_ROOT;
  $FORM{'servername'} = $SERVER_NAME;

  &template("admin.stub.pl","admin/stub.pl"); 

####################################################
# update WebClass log pages   

### log files now opened for APPEND in batch.pl ###
# print LOG "$date~$term~$dept~$num~$act~$sect~$prof~$srs~$classurl\n";
## Need log to mysql table

    if ($batchmode) {
      print "$classdir  ";
    } else { 
      print "<html>\n <head>\n  <title>$_CLASSWEBSITEMADE...</title>\n";
      print "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"3; URL=$url\"></head>\n";
      print "<BODY BGCOLOR=#FFFFFF><h3>$_CLASSWEBSITEMADE - $_REDIRECTINGYOUTO:</h3>\n";
      print "<a href=\"$url\">$url</a>\n\n\n" ;   # Return URL of new site
      print "</body></html>\n";
    }
    return $url;
  }  # end of write_page subroutine


####################################################
sub error {
  # called inside write_page subroutine
  ($error,@error_fields) = @_;

  require "$BASE/contenttype.inc"; # multilingual content-type
#  print "Content-type: text/html\n\n";
  print "<html>\n <head>\n  <title>$_ERROR: $error</title>\n </head>\n";
  print " </head>\n <BODY BGCOLOR=#FFFFFF>\n";

  print "   <h1>$_ERROR: $error</h1>\n\n";
  foreach $error_field (@error_fields) {
    print "$error_field\n";
  }

# added by Artin and Mike
  print "$_BROWSERLEFTARROW $CONTACT_ADMIN\n";
  print "</body></html>\n";
  exit;
}    # end of error subroutine


####################################################
# use input templates combined with $FORM vars to write out file
sub template {
  # called inside write_page subroutine
  my($in,$out) = @_;
    if ($customsetup{$dept} && -e "$customsetup{$dept}/$in") {
      open(MODEL,"<$customsetup{$dept}/$in");
    } else {
      open(MODEL,"<$PROFBASE/$in");
    }
    open (HOME, ">$outdir/$out") || 
      &error("$out","Couldn't open $outdir/$out"); 

    # this section replaces the #=codes=# with variables from FORM hash
    while (<MODEL>) {
      s/#=(.*?)=#/$FORM{$1}/g;   # variable between #= and =#
      print HOME;
    }
    close(MODEL);
    close(HOME);
    if ($in =~ /\.pl$/) {   # perl scripts need to be executable
      chmod (0770, "$outdir/$out");
    } else {
      chmod (0660, "$outdir/$out");
    }

}  # end of template subroutine

####################################################
# log to profcontacts table
sub log_profcontact {
  # called inside send_email subroutine
  my($uid,$note,$followup) = @_;
  $cmd = "insert into profcontacts (uid,type,notes,followup)
    values('$uid','massmail','$note','$followup')";
    $sth2 = $dbh->prepare($cmd) or die "Can't prepare $cmd: $dbh->errstr\n";
    $sth2->execute or die "Can't execute query: $sth->errstr\n";
}
}    # end of main setup subroutine

